<?php

namespace Drupal\static_builder_gatsby\Plugin\static_build\StaticBuilder;

use Drupal\static_build\Plugin\StaticBuilderPluginBase;
use Drupal\static_suite\StaticSuiteException;

/**
 * Provides a static builder for Gatsby.
 *
 * @StaticBuilder(
 *  id = "gatsby",
 *  label = @Translation("Gatsby"),
 *  description = @Translation("Static builder to generate sites using Gatsby"),
 *  host = "local"
 * )
 */
class GatsbyStaticBuilder extends StaticBuilderPluginBase {
  protected const GATSBY_CACHE_DIRNAME = '.cache';
  protected const GATSBY_PUBLIC_DIRNAME = 'public';

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function preBuild(): void {
    $this->deleteTempDirs();
  }

  /**
   * {@inheritdoc}
   */
  public function build(): void {
    $config = $this->configFactory->get('static_builder_gatsby.settings');

    // Run command and pipe stderr to stdout.
    $commands = [
      'cd ' . $this->getBuildDir(),
    ];
    if ($this->getConf('run-mode') === self::RUN_MODE_PREVIEW) {
      // @see https://www.gatsbyjs.org/docs/environment-variables/
      putenv('GATSBY_IS_PREVIEW=true');
      $commands[] = 'node ' . $config->get('node.preview.options') . ' node_modules/.bin/gatsby build ' . $config->get('gatsby.preview.options') . ' 2>&1';
    }
    else {
      $commands[] = 'node ' . $config->get('node.live.options') . ' node_modules/.bin/gatsby build ' . $config->get('gatsby.live.options') . ' 2>&1';
    }
    $command = implode(' && ', $commands);

    $this->logMessage('[GATSBY] Executing command: ' . $command);

    $trace = "";
    $handle = popen($command, 'r');
    while ($line = fgets($handle)) {
      $this->logMessage('[GATSBY] ' . trim($line));
      $trace .= $line;
    }
    $exitCode = pclose($handle);
    putenv('GATSBY_IS_PREVIEW');

    if ($exitCode !== 0) {
      throw new StaticSuiteException("Unable to build live. Exit code $exitCode. Trace: $trace");
    }
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function postBuild(): void {
    // Move public dir contents to release dir.
    $this->release->moveToDir($this->getBuildDir() . "/" . self::GATSBY_PUBLIC_DIRNAME, "");

    // Remove .cache and public dirs.
    $this->deleteTempDirs();
  }

  /**
   * Deletes .cache and public dir.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  protected function deleteTempDirs(): void {
    $config = $this->configFactory->get('static_builder_gatsby.settings');
    // Remove .cache and public dirs.
    if ($config->get('delete.cache')) {
      $this->deleteInsideBuildDir(self::GATSBY_CACHE_DIRNAME);
    }
    if ($config->get('delete.public')) {
      $this->deleteInsideBuildDir(self::GATSBY_PUBLIC_DIRNAME);
    }
  }

}
