<?php

namespace Drupal\static_build\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\static_build\Plugin\StaticBuilderHelperInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Throwable;

/**
 * Provides various controllers to interact with releases on build context.
 */
class ReleaseController extends ControllerBase {

  /**
   * The static builder plugin manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface
   */
  protected $staticBuilderPluginManager;

  /**
   * The static builder helper.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderHelperInterface
   */
  protected $staticBuilderHelper;

  /**
   * The static builder plugin.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderPluginInterface
   */
  protected $builder;

  /**
   * StaticBuilderReleasesController constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface $staticBuilderPluginManager
   *   The static builder plugin manager.
   * @param \Drupal\static_build\Plugin\StaticBuilderHelperInterface $staticBuilderHelper
   *   The static builder helper.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    StaticBuilderPluginManagerInterface $staticBuilderPluginManager,
    StaticBuilderHelperInterface $staticBuilderHelper
  ) {
    $this->configFactory = $config_factory;
    $this->staticBuilderPluginManager = $staticBuilderPluginManager;
    $this->staticBuilderHelper = $staticBuilderHelper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.static_builder'),
      $container->get('static_build.static_builder_helper')
    );
  }

  /**
   * Shows a list of releases.
   *
   * @param string $builderId
   *   Builder id.
   * @param string $runMode
   *   A run mode to get releases for. Usually live or preview.
   *
   * @return array
   *   A render array as expected by the renderer.
   */
  public function listReleases(string $builderId, string $runMode): array {
    $builder = NULL;
    try {
      $builder = $this->staticBuilderPluginManager->getInstance([
        'plugin_id' => $builderId,
        'configuration' => ['run-mode' => $runMode],
      ]);
      $releaseManager = $builder->getReleaseManager();
    }
    catch (Throwable $e) {
      return $list['static_build_releases_list'] = [
        '#markup' => '<p>' . $this->t('Unable to list releases:') . ' ' . $e->getMessage() . '</p>',
      ];
    }

    $allReleases = $releaseManager->getAllReleases();

    $rows = [];
    $ok = json_decode('"\u2714\ufe0f"', FALSE);
    $ko = json_decode('"\u274c"', FALSE);
    foreach ($allReleases as $release) {
      $releaseTask = $release->task($builder->getTaskId());
      $row = [];
      $row['id'] = $release->uniqueId();
      $row['speed'] = $releaseTask->getProcessBenchmark() . ' ' . $this->t('seconds');
      $row['percentage'] = $releaseManager->getTaskSupervisor()->getTaskPercentage($release->uniqueId(), $builder->getTaskId()) . "%";
      $row['current'] = $releaseManager->isCurrent($release->uniqueId()) ? $ok : $ko;
      $row['build-done'] = $releaseTask->isDone() ? $ok : $ko;
      $row['build-log']['data'] = [
        '#type' => 'inline_template',
        '#template' => '<a href="' . Url::fromRoute('static_build.log_viewer', [
          'builderId' => $builderId,
          'runMode' => $runMode,
          'uniqueId' => $release->uniqueId(),
        ])->toString() . '" target="_blank">' . $this->t('View') . '</a>',
      ];
      $rows[] = $row;
    }

    $headers = [
      $this->t('Unique ID'),
      $this->t('Build speed (~@medianSpeed secs)', ['@medianSpeed' => $releaseManager->getTaskSupervisor()->getMedianTaskTime($builder->getTaskId())]),
      $this->t('Build percentage'),
      $this->t('Is current'),
      $this->t('Successfully built'),
      $this->t('Build log'),
    ];

    $headerMessage = $this->t(
        'Releases built by <strong>"@builder"</strong> on <strong>"@run-mode"</strong> mode.',
        [
          '@builder' => $builder->label(),
          '@run-mode' => $runMode,
          '@cloud' => StaticBuilderPluginInterface::HOST_MODE_CLOUD,
        ]
      );

    $subHeaderMessage = NULL;
    if ($runMode === StaticBuilderPluginInterface::RUN_MODE_PREVIEW) {
      $subHeaderMessage = $this->t(
        'Keep in mind that "@cloud" builders do not support "@run-mode" mode.',
        [
          '@cloud' => StaticBuilderPluginInterface::HOST_MODE_CLOUD,
          '@run-mode' => $runMode,
        ]
      );
    }

    $list['releases_list_header'] = [
      '#markup' => '<p>' . $headerMessage . ' ' . $subHeaderMessage . '</p>',
    ];

    $list['releases_list'] = [
      '#type' => 'table',
      '#header' => $headers,
      '#rows' => $rows,
      '#empty' => $this->t('No releases available. Please edit some content or run a build manually.'),
      '#attributes' => [
        'class' => ['block-add-table'],
      ],
    ];

    return $list;
  }

  /**
   * Get data from release currently being built.
   *
   * Gives an estimated percentage.
   *
   * @param string $builderId
   *   Builder id.
   * @param string $runMode
   *   A run mode. Usually live or preview.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response.
   */
  public function getRunningReleaseData(string $builderId, string $runMode): JsonResponse {
    return new JsonResponse(
      $this->staticBuilderHelper->getRunningBuildData($builderId, $runMode)
    );
  }

}
