<?php

namespace Drupal\static_build\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\node\Entity\Node;
use Drupal\static_build\Plugin\StaticBuilderPluginInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface;
use Drupal\static_export\Exporter\Type\Entity\EntityExporterPluginInterface;
use Drupal\static_suite\Entity\EntityUtils;
use Drupal\static_export\Event\StaticExportEvent;
use Drupal\static_export\Event\StaticExportEvents;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for Static Build.
 */
class EventSubscriber implements EventSubscriberInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Static Builder Manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface
   */
  protected $staticBuilderPluginManager;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtils
   */
  protected $entityUtils;

  /**
   * Static Export utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * Constructs the subscriber.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface $staticBuilderPluginManager
   *   Static Builder Manager.
   * @param \Drupal\static_suite\Entity\EntityUtils $entityUtils
   *   Utils for working with entities.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $static_suite_utils
   *   Static Export utils.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StaticBuilderPluginManagerInterface $staticBuilderPluginManager, EntityUtils $entityUtils, StaticSuiteUtilsInterface $static_suite_utils) {
    $this->configFactory = $config_factory;
    $this->staticBuilderPluginManager = $staticBuilderPluginManager;
    $this->entityUtils = $entityUtils;
    $this->staticSuiteUtils = $static_suite_utils;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // TODO - Instead of listening to WRITE_QUEUE_PROCESSING_ENDS event, we
    //  should listen the "WRITE_ENDS" event once writing to stdout has been
    //  removed. The idea behind using WRITE_QUEUE_PROCESSING_ENDS was that in
    //  this moment, a data semaphore is active, so we could start a build, and
    //  that build can reuse that data semaphore to copy data files.
    //  But, in fact, a build doesn't reuse any semaphore but creates a new one
    //  because most build processes happen asynchronously. Due to being async,
    //  when the build process creates a data semaphore, the static_export data
    //  semaphore is already released, and in the meanwhile some other export
    //  has happened or is happening. Even though this could seem like a bug,
    //  in fact it isn't because this way the build process is agnostic and
    //  always uses the last data it can found. If two concurrent export
    //  processes trigger two builds, and the first one get the data exported
    //  by the second one, the second build won't be executed since there is
    //  a check to ensure data with the same UNIQUE_ID is not built twice.
    $events[StaticExportEvents::WRITE_QUEUE_PROCESSING_ENDS][] = ['requestLiveBuild'];
    return $events;
  }

  /**
   * Reacts to a StaticExportEvents::WRITE_QUEUE_PROCESSING_ENDS event.
   *
   * @param \Drupal\static_export\Event\StaticExportEvent $event
   *   The Static Export event.
   *
   * @return \Drupal\static_export\Event\StaticExportEvent
   *   The processed event.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function requestLiveBuild(StaticExportEvent $event): StaticExportEvent {
    // Check if received event should request a build.
    $exporter = $event->getExporter();
    if (!$exporter->mustBuild()) {
      $exporter->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_LIVE . ' | Exporter is not allowed to request a build.');
      return $event;
    }

    $localBuilders = $this->configFactory->get('static_build.settings')->get('local.builders');
    $cloudBuilders = $this->configFactory->get('static_build.settings')->get('cloud.builders');
    $allBuilders = [
      StaticBuilderPluginInterface::HOST_MODE_LOCAL => $localBuilders,
      StaticBuilderPluginInterface::HOST_MODE_CLOUD => $cloudBuilders,
    ];
    foreach ($allBuilders as $hostMode => $builders) {
      if (is_array($builders)) {
        foreach ($builders as $builderId) {
          $staticBuilder = $this->staticBuilderPluginManager->getInstance([
            'plugin_id' => $builderId,
            'configuration' => [
              'run-mode' => StaticBuilderPluginInterface::RUN_MODE_LIVE,
              'console-output' => '',
              // Only request a deploy for local builders.
              'request-deploy' => $hostMode === StaticBuilderPluginInterface::HOST_MODE_LOCAL,
            ],
          ]);

          // Check if entity is published or status has changed.
          if ($exporter instanceof EntityExporterPluginInterface) {
            $entity = $exporter->getExporterItem();

            // Check if it's published.
            $isPublished = TRUE;
            if ($entity instanceof Node) {
              $isPublished = $entity->isPublished();
            }

            // Check if status has changed: if status is unpublished, but status
            // has changed (from published to unpublished, or vice versa) we
            // must build it.
            $statusHasChanged = FALSE;
            if ($entity instanceof EditorialContentEntityBase) {
              $statusHasChanged = $this->entityUtils->hasStatusChanged($entity);
            }

            if (!$isPublished && !$statusHasChanged) {
              $event->getExporter()
                ->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_LIVE . " | Exporter entity is not published nor status has changed. No build will we triggered.");
              return $event;
            }
          }

          // Check if received FileCollections contains changes.
          $fileCollectionGroup = $event->getFileCollectionGroup();
          if (!$fileCollectionGroup || !$fileCollectionGroup->isAnyFileCollectionExecuted()) {
            $exporter->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_LIVE . ' | No changes detected on received FileCollectionGroup. No build will we triggered.');
            return $event;
          }

          // Check if received FileCollections matches paths that request a build.
          $executedFilePaths = $fileCollectionGroup->getExecutedFilePathsFromAllFileCollections();
          $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_LIVE . " | Changed files on received FileCollectionGroup:\n" . implode("\n", $executedFilePaths));
          $changedFilesRegExpList = $this->getBuildTriggerRegExpList();
          $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_LIVE . " | RegExp List to match changed files:\n" . implode("\n", $changedFilesRegExpList));
          if (is_array($changedFilesRegExpList) && count($changedFilesRegExpList) && !$this->staticSuiteUtils->isAnyItemMatchingRegexpList($executedFilePaths, $changedFilesRegExpList)) {
            $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_LIVE . ' | No changed files match RegExp List. No build triggered.');
            return $event;
          }

          $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_LIVE . ' | Changes detected on received FileCollectionGroup. Build triggered.');

          // Finally, trigger a build.
          $staticBuilder->init();

        }
      }
    }

    return $event;
  }

  /**
   * Get build_trigger_regexp_list config value.
   *
   * @return array
   *   Array with Regular Expressions or null otherwise.
   */
  protected function getBuildTriggerRegExpList(): array {
    $config = $this->configFactory->get('static_build.settings');
    $regExpList = $config->get('build_trigger_regexp_list_live');
    if (is_array($regExpList)) {
      return $regExpList;
    }
    return [];
  }

}
