<?php

namespace Drupal\static_build\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\static_build\Event\StaticBuildEvent;
use Drupal\static_build\Event\StaticBuildEvents;
use Drupal\static_build\Release\Release;
use Drupal\static_build\Release\ReleaseInterface;
use Drupal\static_build\Release\ReleaseManagerInterface;
use Drupal\static_export\Exporter\ExporterPluginInterface;
use Drupal\static_export\Exporter\ExporterReporterInterface;
use Drupal\static_suite\Lock\LockBackendInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Drupal\static_suite\Utility\UniqueIdHelperInterface;
use Drupal\static_export\File\FileCollectionWriter;
use Drupal\static_suite\StaticSuiteException;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Throwable;

/**
 * Base class for Static Builder plugins.
 */
abstract class StaticBuilderPluginBase extends PluginBase implements StaticBuilderPluginInterface, ContainerFactoryPluginInterface {
  use LoggerChannelTrait;
  use StringTranslationTrait;

  /**
   * Unique identifier.
   *
   * @var string
   */
  protected $uniqueId;

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Drupal file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Release Manager.
   *
   * @var \Drupal\static_build\Release\ReleaseManagerInterface
   */
  protected $releaseManager;

  /**
   * Exporter service.
   *
   * @var \Drupal\static_export\Exporter\ExporterReporterInterface
   */
  protected $exporterReporter;

  /**
   * Static Export utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * Unique ID helper.
   *
   * @var \Drupal\static_suite\Utility\UniqueIdHelperInterface
   */
  protected $uniqueIdHelper;

  /**
   * Release being built.
   *
   * @var \Drupal\static_build\Release\Release
   */
  protected $release;

  /**
   * Release task.
   *
   * @var \Drupal\static_build\Release\Task\Task
   */
  protected $releaseTask;

  /**
   * Array of values used to check if we must build.
   *
   * Used to avoid infinite loops.
   *
   * @var array
   */
  protected $mustBuildCheckValues;

  /**
   * Index number during the build loop.
   *
   * @var int
   */
  protected $buildIndex;

  /**
   * The lock system from Static Suite.
   *
   * @var \Drupal\Core\Lock\LockBackendInterface
   */
  protected $lock;

  /**
   * The data dir from static_export.
   *
   * @var string
   */
  protected $dataDir;

  /**
   * An array of messages to be logged.
   *
   * @var array
   */
  protected $logMessagesStack;

  /**
   * Time start.
   *
   * @var float
   */
  protected $timeStart;

  /**
   * Time end.
   *
   * @var float
   */
  protected $timeEnd;

  /**
   * {@inheritdoc}
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config service.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   Drupal file system service.
   * @param \Drupal\static_suite\Lock\LockBackendInterface $lock
   *   The lock system from Static Suite.
   * @param \Drupal\static_build\Release\ReleaseManagerInterface $releaseManager
   *   The release manager.
   * @param \Drupal\static_export\Exporter\ExporterReporterInterface $exporterReporter
   *   The exporter reporter service.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $static_suite_utils
   *   Static Export utils.
   * @param \Drupal\static_suite\Utility\UniqueIdHelperInterface $unique_id_helper
   *   Unique ID helper.
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $configFactory,
    EventDispatcherInterface $eventDispatcher,
    FileSystemInterface $fileSystem,
    LockBackendInterface $lock,
    ReleaseManagerInterface $releaseManager,
    ExporterReporterInterface $exporterReporter,
    StaticSuiteUtilsInterface $static_suite_utils,
    UniqueIdHelperInterface $unique_id_helper
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $configFactory;
    $this->eventDispatcher = $eventDispatcher;
    $this->fileSystem = $fileSystem;
    $this->lock = $lock;
    $this->setConfiguration($configuration);
    $this->setDerivedConfiguration();

    $this->dataDir = $this->configFactory->get('static_export.settings')->get('data_dir');
    $this->releaseManager = $releaseManager;
    $this->exporterReporter = $exporterReporter;
    $this->staticSuiteUtils = $static_suite_utils;
    $this->uniqueIdHelper = $unique_id_helper;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public static function create(ContainerInterface $container,
                                array $configuration,
                                $plugin_id,
                                $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get("config.factory"),
      $container->get("event_dispatcher"),
      $container->get("file_system"),
      $container->get("static_suite.lock"),
      $container->get("static_build.release_manager"),
      $container->get("static_export.exporter_reporter"),
      $container->get("static_suite.utils"),
      $container->get("static_export.unique_id_helper")
    );
  }

  /**
   * {@inheritdoc}
   */
  public function uniqueId(): string {
    return $this->uniqueId;
  }

  /**
   * {@inheritdoc}
   */
  public function getTaskId(): string {
    return self::TASK_ID . '-' . $this->getConf('id');
  }

  /**
   * Get dir where releases get built.
   *
   * @return string
   *   The dir where releases get built.
   */
  public function getBuildDir(): string {
    return $this->getConf('build-dir');
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function init(): bool {
    // Fork an async process only if not in a cli and async option is true.
    if (!$this->staticSuiteUtils->isRunningOnCli() && $this->getConf('sync') === FALSE) {
      // Define a log file so we can get info about the forking process.
      $forkLog = $this->fileSystem->getTempDirectory() . '/static_build_fork.' . $this->getConf('run-mode') . '.' . $this->getConf('id');
      $forkLog .= $this->getConf('drush-options') ? '.' . md5($this->getConf('drush-options')) : '';
      $forkLog .= '.log';

      $commandOptions = [
        $this->getConf('id'),
        $this->getConf('run-mode'),
        '--lock-mode=' . $this->getConf('lock-mode'),
        $this->getConf('drush-options'),
      ];

      // Run async command.
      $asyncCommands = array_merge(
        $this->getEnvironmentVariables(),
        [
          'cd ' . DRUPAL_ROOT,
          self::DRUSH_ASYNC_COMMAND . ' ' . implode(' ', $commandOptions) . ' > ' . $forkLog . ' 2>&1 &',
        ]
      );

      $this->dispatchEvent(StaticBuildEvents::ASYNC_PROCESS_FORK_START);
      exec(implode(' && ', $asyncCommands), $output, $returnValue);
      $this->dispatchEvent(StaticBuildEvents::ASYNC_PROCESS_FORK_END);
      if ($returnValue !== 0) {
        throw new StaticSuiteException("Unable to fork build process:\n" . implode("\n", $output));
      }

      return TRUE;
    }

    // Otherwise, run it synchronously.
    return $this->runBuildLoop();
  }

  /**
   * Get environment variables for forked process.
   *
   * @return array
   *   Array with environment commands.
   */
  protected function getEnvironmentVariables(): array {
    $env = $this->getConf('env');
    if (!is_array($env)) {
      $env = [];
    }

    // Search for $HOME and $PATH inside $env.
    $isHomeDefined = FALSE;
    $isPathDefined = FALSE;
    foreach ($env as $envLine) {
      if (preg_match("/^\s*export\s*HOME=/", $envLine)) {
        $isHomeDefined = TRUE;
      }
      elseif (preg_match("/^\s*export\s*PATH=/", $envLine)) {
        $isPathDefined = TRUE;
      }
    }

    // Set default $HOME and $PATH if not defined in $env.
    if (!$isHomeDefined) {
      $env[] = 'export HOME=' . $this->staticSuiteUtils->discoverDefaultHome();
    }
    if (!$isPathDefined) {
      $env[] = 'export PATH=/usr/local/bin:/usr/bin:/bin';
    }

    $environmentCommands = [];
    if (is_array($env)) {
      foreach ($env as $envLine) {
        // Ensure only lines starting with "export" are executed.
        if (preg_match("/^\s*export\s+/", $envLine)) {
          $environmentCommands[] = $envLine;
        }
      }
    }

    return $environmentCommands;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function runBuildLoop(): bool {
    $this->dispatchEvent(StaticBuildEvents::START);
    $this->buildIndex = 1;
    $buildLockName = $this->getConf('run-mode-dir') . '--build';
    try {
      // First of all, create log dir.
      $this->createLogDir();

      $this->releaseManager->init($this->getConf('run-mode-dir'));

      $this->logMessage("**********************************************************");
      $this->logMessage("BUILD RUN STARTS");
      $this->logMessage("**********************************************************");
      $this->dispatchEvent(StaticBuildEvents::BUILD_RUN_START);
      // Unset console-output to avoid getting that object printed in logs.
      $configurationWithoutConsoleOutput = $this->configuration;
      unset($configurationWithoutConsoleOutput['console-output']);
      $this->logMessage("Config:\n" . print_r($configurationWithoutConsoleOutput, TRUE));
      $this->logMessage("ENV:\n" . print_r($_ENV, TRUE));

      if (!$this->lock->acquire($buildLockName, $this->configFactory->get('static_build.settings')->get('semaphore_timeout'))) {
        $this->logMessage("Another build process is running. Aborting this build.");
        $this->logMessage("**********************************************************");
        $this->logMessage("BUILD RUN ABORTED");
        $this->logMessage("**********************************************************\n");
        return FALSE;
      }

      $this->logMessage("[LOCK] Build lock acquired.");
      $this->logMessage("No other build process running. Go ahead with this build.");

      // Check if we really need to build the site again on each iteration.
      while (TRUE) {
        // Lock data-dir to ensure next checks (mainly mustBuild()) operate on
        // data that is 100% accurate.
        $this->logMessage("[LOCK] Waiting to lock data dir...");
        if (!$this->lock->acquireOrWait($this->dataDir)) {
          throw new StaticSuiteException('Could not acquire lock on "' . $this->dataDir . '", timeout reached.');
        }
        $this->logMessage("[LOCK] Data dir lock successfully acquired.");

        // Discover "last locked executed unique id" and set it as our
        // Unique ID for this loop.
        $this->uniqueId = $this->getLastLockExecutedUniqueId();
        $this->logMessage("Discovered 'last locked executed unique id': " . $this->uniqueId);
        $this->logMessage("Using this UNIQUE ID for this loop #" . $this->buildIndex . ": " . $this->uniqueId);

        // Check if we need to run a build.
        if (!$this->mustBuild()) {
          $this->lock->release($this->dataDir);
          $this->logMessage("[LOCK] Data dir lock released.");
          $this->logMessage("Breaking the loop...");
          break;
        }

        $this->logMessage("----------------------------------------------------------");
        $this->logMessage("[" . $this->buildIndex . "] BUILD LOOP #" . $this->buildIndex . " STARTS");
        $this->logMessage("----------------------------------------------------------");

        $this->dispatchEvent(StaticBuildEvents::BUILD_LOOP_START);

        $this->startBenchmark();

        // CREATE RELEASE DIR.
        $this->dispatchEvent(StaticBuildEvents::RELEASE_DIR_CREATION_START);
        $this->logMessage("[" . $this->buildIndex . "] Creating release dir...");
        $this->release = $this->releaseManager->create($this->uniqueId());
        $this->release->createReleaseDir();
        $this->releaseTask = $this->release->task($this->getTaskId());
        $this->releaseTask->setStarted();
        $this->logMessage("[" . $this->buildIndex . "] Release dir created at " . $this->release->getDir());
        $this->dispatchEvent(StaticBuildEvents::RELEASE_DIR_CREATION_END);

        // COPY DATA-DIR.
        $this->dispatchEvent(StaticBuildEvents::DATA_COPYING_START);
        if (!is_dir($this->getConf('build-dir'))) {
          mkdir($this->getConf('build-dir'), 0777, TRUE);
        }
        $this->logMessage("[" . $this->buildIndex . "] Copying data dir " . $this->dataDir . ' to ' . $this->getConf('build-dir'));
        if ($this->isLocal()) {
          $this->copyDataToBuildDir($this->dataDir, 'data', $this->getDataDirExcludedPaths());
        }
        else {
          $this->copyDataToBuildDir($this->dataDir, 'data', $this->getDataDirExcludedPaths(), TRUE);
        }
        $this->logMessage("[" . $this->buildIndex . "] Data dir copied.");
        $this->dispatchEvent(StaticBuildEvents::DATA_COPYING_END);

        // RELEASE DATA-DIR LOCK.
        $this->lock->release($this->dataDir);
        $this->logMessage("[" . $this->buildIndex . "] [LOCK] Data dir lock released. Don't panic if you see a concurrent build trying to start and giving up.");

        // EXECUTE PREBUILD.
        $this->dispatchEvent(StaticBuildEvents::PRE_BUILD_START);
        $this->logMessage("[" . $this->buildIndex . "] Executing preBuild for " . $this->release->getDir());
        $this->preBuild();
        $this->logMessage("[" . $this->buildIndex . "] Prebuild executed.");
        $this->dispatchEvent(StaticBuildEvents::PRE_BUILD_END);

        // BUILD.
        $this->dispatchEvent(StaticBuildEvents::BUILD_START);
        $this->logMessage("[" . $this->buildIndex . "] Executing build on mode " . $this->getConf('run-mode') . " for " . $this->release->getDir());
        $this->build();
        $this->logMessage("[" . $this->buildIndex . "] build executed.");
        $this->dispatchEvent(StaticBuildEvents::BUILD_END);

        // EXECUTE POST BUILD.
        $this->dispatchEvent(StaticBuildEvents::POST_BUILD_START);
        $this->logMessage("[" . $this->buildIndex . "] Executing postBuild for " . $this->release->getDir());
        $this->postBuild();
        $this->logMessage("[" . $this->buildIndex . "] PostBuild executed.");
        $this->dispatchEvent(StaticBuildEvents::POST_BUILD_END);

        // Mark this release as successfully built.
        $this->releaseTask->setDone();

        // PUBLISH RELEASE.
        $this->dispatchEvent(StaticBuildEvents::PUBLISH_RELEASE_START);
        $this->logMessage("[" . $this->buildIndex . "] Trying to publish release " . $this->release->uniqueId());
        $this->setCurrentSymlink();
        $this->logMessage("[" . $this->buildIndex . "] Release published.");
        $this->dispatchEvent(StaticBuildEvents::PUBLISH_RELEASE_END);

        // DELETE OLD RELEASES.
        $this->dispatchEvent(StaticBuildEvents::OLD_RELEASES_DELETION_START);
        $this->logMessage("[" . $this->buildIndex . "] Deleting old releases.");
        $this->releaseManager->deleteOldReleases();
        $this->logMessage("[" . $this->buildIndex . "] Old releases deleted.");
        $this->dispatchEvent(StaticBuildEvents::OLD_RELEASES_DELETION_END);

        $this->logMessage("----------------------------------------------------------");
        $this->logMessage("[" . $this->buildIndex . "] BUILD LOOP #" . $this->buildIndex . " ENDS");
        $this->logMessage("----------------------------------------------------------");

        $this->dispatchEvent(StaticBuildEvents::BUILD_LOOP_END);

        $this->getBenchmark();

        $this->buildIndex++;
      }

      // RELEASE BUILD LOCK.
      $this->lock->release($buildLockName);
      $this->logMessage("[LOCK] Build lock released.");
      $this->logMessage("**********************************************************");
      $this->logMessage("BUILD RUN ENDS");
      $this->logMessage("**********************************************************\n");
      $this->dispatchEvent(StaticBuildEvents::BUILD_RUN_END);

      $this->dispatchEvent(StaticBuildEvents::END);
      return TRUE;
    }
    catch (Throwable $e) {
      // This is Throwable to capture any kind of exception and ensure
      // that any pending work is done.
      $this->logMessage("Exception thrown. Cleaning up...");
      $this->logMessage($e->getMessage() . "\n" . $e->getTraceAsString());

      // If this exception comes from Static Build, log an error.
      if ($e instanceof StaticSuiteException) {
        $message = $e->getMessage() . " Please, review " . $this->getConf('revisions-log-file');
        $this->getLogger('static_build')->error($message);
      }

      // Mark release as failed.
      if ($this->releaseTask && $this->releaseTask->isStarted()) {
        $this->releaseTask->setFailed();
      }

      // Double ensure that a current release is present.
      if (!$this->releaseManager->getCurrentUniqueId()) {
        $this->logMessage("[" . $this->buildIndex . "] No current release. This is a major issue. Trying to fix it.");
        // Find the latest done release and do a rollback.
        $lastDoneRelease = $this->releaseManager->getTaskSupervisor()->getLastDoneRelease($this->getTaskId());
        if ($lastDoneRelease && $lastDoneRelease->uniqueId()) {
          $lastTryResult = $this->releaseManager->publish($lastDoneRelease->uniqueId(), $this->getTaskId());
          if ($lastTryResult === TRUE) {
            $this->logMessage("LAST ROLLBACK FOR RELEASE " . $lastDoneRelease->uniqueId() . " SUCCESSFUL!");
          }
          else {
            // OH NO!!! WE CANNOT FIX THIS!!!
            $this->logMessage("LAST ROLLBACK FOR RELEASE " . $lastDoneRelease->uniqueId() . " FAILED!");
            $this->logMessage("[" . $this->buildIndex . "] ######################################################");
            $this->logMessage("[" . $this->buildIndex . "] ################### SITE IS BROKEN ###################");
            $this->logMessage("[" . $this->buildIndex . "] ######################################################");

          }
        }
      }

      // Delete old releases to protect disk space if builds are falling and
      // they are been running continuously.
      $this->logMessage("[" . $this->buildIndex . "] Deleting old releases.");
      $this->releaseManager->deleteOldReleases();
      $this->logMessage("[" . $this->buildIndex . "] Old releases deleted.");

      // Release any pending lock.
      $this->lock->release($this->dataDir);
      $this->logMessage("[LOCK] Data lock released.");
      $this->lock->release($buildLockName);
      $this->logMessage("[LOCK] Build lock released.");

      $this->dispatchEvent(StaticBuildEvents::END);
      throw new StaticSuiteException($e);
    }
    finally {
      $this->lock->releaseAll();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preBuild(): void {
    // By default, don't do nothing.
  }

  /**
   * {@inheritdoc}
   *
   * TODO allow to change this setting from config.
   *  This value comes from Static Export config, so get it from Drupal config
   *  and let adding more values.
   */
  public function getDataDirExcludedPaths(): array {
    return [
      ExporterPluginInterface::WORK_DIR,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function postBuild(): void {
    // By default, don't do nothing.
  }

  /**
   * Set $this->release as current.
   *
   * Tries to leave the site in good condition under any circumstances, making
   * the current symlink point to the best release found.
   *
   * @return bool
   *   True on success.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  protected function setCurrentSymlink(): bool {
    // Get current id in case we need to execute a rollback.
    $currentReleaseUniqueId = $this->releaseManager->getCurrentUniqueId();
    $this->logMessage("[" . $this->buildIndex . "] Publishing release *** " . $this->release->uniqueId() . " *** . Previous current was $currentReleaseUniqueId.");
    $result = $this->releaseManager->publish($this->release->uniqueId(), $this->getTaskId());
    if ($result === TRUE) {
      $this->logMessage("[" . $this->buildIndex . "] RELEASE PUBLISHED!");
    }
    else {
      $this->logMessage("[" . $this->buildIndex . "] ****** RELEASE NOT PUBLISHED ******");
      $rollbackResult = FALSE;
      // Try with $currentReleaseUniqueId if it's not null.
      if ($currentReleaseUniqueId) {
        $this->logMessage("[" . $this->buildIndex . "] Rolling back to $currentReleaseUniqueId");
        $rollbackResult = $this->releaseManager->publish($currentReleaseUniqueId, $this->getTaskId());
      }
      else {
        $this->logMessage("[" . $this->buildIndex . "] No previous current unique id.");
      }
      if ($rollbackResult === TRUE) {
        $this->logMessage("[" . $this->buildIndex . "] ROLLBACK FOR RELEASE $currentReleaseUniqueId SUCCESSFUL!");
      }
      else {
        $this->logMessage("[" . $this->buildIndex . "] ROLLBACK FOR RELEASE $currentReleaseUniqueId FAILED!");
        $this->logMessage("[" . $this->buildIndex . "] Last try. Ensure that there is a current release.");
        if ($this->releaseManager->getCurrentUniqueId()) {
          $this->logMessage("[" . $this->buildIndex . "] There is a current release for " . $this->releaseManager->getCurrentUniqueId() . '. No need to fix anything.');
        }
        else {
          $this->logMessage("[" . $this->buildIndex . "] No current release. This is a major issue. Trying to fix it.");
          // As a last try, find the latest done release and do a rollback.
          $lastDoneRelease = $this->releaseManager->getTaskSupervisor()->getLastDoneRelease($this->getTaskId());
          $lastTryResult = FALSE;
          if ($lastDoneRelease && $lastDoneRelease->uniqueId()) {
            $lastTryResult = $this->releaseManager->publish($lastDoneRelease->uniqueId(), $this->getTaskId());
          }
          if ($lastTryResult === TRUE) {
            $this->logMessage("[" . $this->buildIndex . "] ROLLBACK FOR RELEASE " . $lastDoneRelease->uniqueId() . " SUCCESSFUL!");
          }
          else {
            // OH NO!!! WE CANNOT FIX THIS!!!
            $this->logMessage("[" . $this->buildIndex . "] ROLLBACK FOR RELEASE " . $lastDoneRelease->uniqueId() . " FAILED!");
            $this->logMessage("[" . $this->buildIndex . "] ######################################################");
            $this->logMessage("[" . $this->buildIndex . "] ################### SITE IS BROKEN ###################");
            $this->logMessage("[" . $this->buildIndex . "] ######################################################");

            // Throw this error.
            throw new StaticSuiteException("SITE IS BROKEN. NO CURRENT SYMLINK. Unable to publish release " . $this->release->uniqueId());
          }
        }
      }
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function mustBuild(): bool {
    $this->logMessage('Checking if we must run a build with the following info:');
    $this->logMessage('$this->uniqueId=' . $this->uniqueId);

    // Check current release unique id.
    $currentRelease = $this->releaseManager->getCurrentRelease();
    $currentReleaseUniqueId = $currentRelease ? $currentRelease->uniqueId() : NULL;
    $this->logMessage('$currentReleaseUniqueId=' . $currentReleaseUniqueId);
    if ($this->uniqueId === $currentReleaseUniqueId) {
      $this->logMessage('$this->uniqueId (' . $this->uniqueId . ") is the same as currentReleaseUniqueId ($currentReleaseUniqueId).");
      $this->logMessage('*** NO NEED TO RUN A BUILD ***');
      return FALSE;
    }

    // Check if $this->uniqueId has a release dir. That means it's a failed
    // build so we need to abort to avoid errors and infinite loops.
    $onGoingReleaseExists = $this->releaseManager->exists($this->uniqueId);
    $this->logMessage('$onGoingReleaseExists=' . $onGoingReleaseExists);
    if ($onGoingReleaseExists) {
      $this->logMessage('Ongoing release ' . $this->uniqueId . " exists.");
      $this->logMessage('*** NO NEED TO RUN A BUILD ***');
      return FALSE;
    }

    // Compare current values with previous ones to avoid infinite loops.
    if (
      $this->mustBuildCheckValues['loopUniqueId'] === $this->uniqueId &&
      $this->mustBuildCheckValues['currentReleaseUniqueId'] === $currentReleaseUniqueId &&
      $this->mustBuildCheckValues['onGoingReleaseExists'] === $onGoingReleaseExists
    ) {
      $this->logMessage('Infinite loop detected. Same values as before.');
      $this->logMessage('*** NO NEED TO RUN A BUILD ***');
      return FALSE;
    }

    // Check if any of the files that have changed since last build
    // must trigger a build.
    $buildTriggerRegexps = $this->getConf('build-trigger-regexp-list');
    $this->logMessage('RegExp List to match against changed files: ' . implode("\n", $buildTriggerRegexps));
    if ($currentReleaseUniqueId && is_array($buildTriggerRegexps) && count($buildTriggerRegexps)) {
      $changedFiles = $this->exporterReporter->getChangedFilesAfter($currentReleaseUniqueId);
      $changedFilePaths = [];
      if ($changedFiles) {
        foreach ($changedFiles as $changedFile) {
          $changedFilePaths[] = $changedFile['absolute-path'];
        }
        $this->logMessage('Changed files since last build: ' . implode("\n", $changedFilePaths));
      }
      else {
        $this->logMessage('No changed files since last build.');
      }
      if ($this->staticSuiteUtils->isAnyItemMatchingRegexpList($changedFilePaths, $buildTriggerRegexps)) {
        $this->logMessage('Some changed files match RegExp list.');
      }
      else {
        $this->logMessage('No changed files match RegExp list.');
        $this->logMessage('*** NO NEED TO RUN A BUILD ***');
        return FALSE;
      }
    }
    else {
      $this->logMessage("No 'build-trigger-regexp-list' config found. Check not executed.");
    }

    // Save current values to compare them on next iteration.
    $this->mustBuildCheckValues = [
      'loopUniqueId' => $this->uniqueId,
      'currentReleaseUniqueId' => $currentReleaseUniqueId,
      'onGoingReleaseExists' => $onGoingReleaseExists,
    ];

    $this->logMessage('All checks passed.');
    $this->logMessage('*** MUST RUN A BUILD ***');
    return TRUE;
  }

  /**
   * Create log dir.
   *
   * This dir must be 777 to allow other processes (like deploy) to log
   * its operations.
   *
   * @return bool
   *   True on success.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  protected function createLogDir(): bool {
    if (!is_dir($this->getConf('log-dir'))) {
      $mkdirResult = @mkdir($this->getConf('log-dir'), 0777, TRUE);
      if ($mkdirResult === FALSE) {
        throw new StaticSuiteException("Unable to create log dir " . $this->getConf('log-dir'));
      }
    }
    return TRUE;
  }

  /**
   * Get the last lock executed unique id.
   *
   * It takes into account the current lock mode, because there are is a
   * different lock file for each run mode.
   *
   * @return string
   *   The last unique id which made changes to data-dir.
   */
  protected function getLastLockExecutedUniqueId(): string {
    $workDir = $this->dataDir . '/' . ExporterPluginInterface::WORK_DIR;
    if ($this->getConf('lock-mode') === self::LOCK_MODE_PREVIEW) {
      $filePath = $workDir . "/" . FileCollectionWriter::LAST_LOCK_EXECUTED_PREVIEW_UNIQUE_ID_FILE;
    }
    else {
      $filePath = $workDir . "/" . FileCollectionWriter::LAST_LOCK_EXECUTED_LIVE_UNIQUE_ID_FILE;
    }
    clearstatcache(TRUE);
    $lastLockExecutedUniqueId = @file_get_contents($filePath);
    if ($lastLockExecutedUniqueId === FALSE) {
      $lastLockExecutedUniqueId = $this->uniqueIdHelper->getDefaultUniqueId();
    }
    return $lastLockExecutedUniqueId;
  }

  /**
   * {@inheritdoc}
   */
  public function logMessage(string $message, bool $mustFormat = TRUE) {
    try {
      if (!empty($message)) {
        if ($mustFormat) {
          $timestamp = $this->staticSuiteUtils->getFormattedMicroDate('Y-m-d H:i:s.u');
          $formattedMessage = getmypid() . " [$timestamp] [$this->uniqueId] $message";
        }
        else {
          $formattedMessage = $message;
        }
        $logFileRevisions = $this->getConf('revisions-log-file');
        if (is_writable(dirname($logFileRevisions))) {
          @file_put_contents($logFileRevisions, "$formattedMessage\n", LOCK_EX);
        }

        // build-log-file inside a release is present at a later moment, so
        // save its logs and write them when log is ready to be used.
        if ($this->releaseTask) {
          $logFileBuild = $this->releaseTask->getLogFilePath();
          if (is_writable(dirname($logFileBuild))) {
            $messageToLog = $formattedMessage;
            if (isset($this->logMessagesStack['build']) && is_array($this->logMessagesStack['build']) && count($this->logMessagesStack['build'])) {
              $messageToLog = implode("\n", $this->logMessagesStack['build']) . $formattedMessage;
              unset($this->logMessagesStack['build']);
            }
            @file_put_contents($logFileBuild, "$messageToLog\n", FILE_APPEND | LOCK_EX);
          }
        }
        else {
          $this->logMessagesStack['build'][] = $formattedMessage;
        }

        if ($this->getConf('console-output')) {
          $this->getConf('console-output')->writeln($formattedMessage);
        }
      }
    }
    catch (Throwable $e) {
      // Do nothing.
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function setConfiguration(array $configuration) {
    // Check for a proper run-mode.
    if (!in_array($configuration['run-mode'], [
      self::RUN_MODE_PREVIEW,
      self::RUN_MODE_LIVE,
    ], TRUE)) {
      throw new StaticSuiteUserException('Run mode "' . $configuration['run-mode'] . '" is not supported. Possible values: ' . self::RUN_MODE_PREVIEW . ' and ' . self::RUN_MODE_LIVE);
    }

    // Ensure that lock-mode defaults to run-mode if no one provided.
    if (empty($configuration['lock-mode'])) {
      $configuration['lock-mode'] = $configuration['run-mode'];
    }

    // Check for a proper lock-mode.
    if (!in_array($configuration['lock-mode'], [
      self::LOCK_MODE_PREVIEW,
      self::LOCK_MODE_LIVE,
    ], TRUE)) {
      throw new StaticSuiteUserException('Lock mode "' . $configuration['lock-mode'] . '" is not supported. Possible values: ' . self::LOCK_MODE_PREVIEW . ' and ' . self::LOCK_MODE_LIVE);
    }

    // Check for a proper lock-mode.
    if (
      $this->isCloud() &&
      $configuration['lock-mode'] === self::LOCK_MODE_PREVIEW
    ) {
      throw new StaticSuiteUserException('Lock mode "' . $configuration['lock-mode'] . '" is not available for cloud builders. Possible values: ' . self::LOCK_MODE_LIVE);
    }

    // Merge config and get a new one.
    $mergedConf = NestedArray::mergeDeep(
      $this->baseConfigurationDefaults($configuration),
      $this->defaultConfiguration(),
      $configuration
    );

    // Finally, set the config.
    $this->configuration = $mergedConf;
  }

  /**
   * Set other config values derived from current ones.
   *
   * This method is executed once base configuration and plugins (Gatsby, Hugo)
   * configuration have been applied. This way, different plugins can define
   * different base directories (base_dir) and be run in parallel.
   */
  protected function setDerivedConfiguration() {
    $this->setConf('run-mode-dir', $this->getConf('base-dir') . '/' . $this->getConf('run-mode'));
    $this->setConf('build-dir', $this->getConf('run-mode-dir') . '/' . self::BUILD_DIR);
    $this->setConf('log-dir', $this->getConf('run-mode-dir') . '/' . self::LOG_DIR);
    $this->setConf('revisions-log-file', $this->getConf('log-dir') . '/' . self::LAST_BUILD_LOG_NAME);
  }

  /**
   * Returns base configuration for static builder plugins.
   *
   * This configuration is defined by static_build module, and must be
   * overridden by any plugin using defaultConfiguration()
   * and setConfiguration().
   *
   * The idea behind this strategy is that StaticBuilderBase can define a base
   * configuration from static_build's configuration form. That can be later
   * overridden by a specific static builder, using defaultConfiguration().
   * Accessing any overridden value is transparent thanks to the use of
   * StaticBuilderBase::getConf()
   *
   * @param array $configuration
   *   An associative array with the configuration passed from outside this
   *   builder.
   *
   * @return array
   *   An associative array with the base configuration.
   */
  protected function baseConfigurationDefaults(array $configuration): array {
    $config = $this->configFactory->get('static_build.settings');

    $baseConf = [
      'id' => $this->getPluginId(),
      'label' => $this->pluginDefinition['label'],
      'description' => $this->pluginDefinition['description'],
      'host' => $this->pluginDefinition['host'],
      'run-mode' => $configuration['run-mode'],
      'lock-mode' => $configuration['lock-mode'],
      'base-dir' => $config->get('base_dir') . '/' . $this->getPluginId(),
      // "number_of_releases_to_keep" is not used through a builder at this
      // moment, so it cannot be overridden by other plugins.
      'drush-env' => $config->get('drush_env'),
      'drush-options' => $config->get('drush_options'),
      'build-request-policy' => $config->get('build_request_policy'),
      'build-trigger-regexp-list' => $config->get('build_trigger_regexp_list_' . $configuration['run-mode']) ?: [],
      'sync' => empty($config->get('sync')) ? FALSE : TRUE,
      'request-deploy' => FALSE,
    ];

    return $baseConf;
  }

  /**
   * Set a conf value.
   *
   * @param string $key
   *   Thd config key.
   * @param mixed $value
   *   Thd config value.
   */
  protected function setConf(string $key, $value) {
    $this->configuration[$key] = $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConf($key) {
    return $this->configuration[$key] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isLocal(): bool {
    return $this->getConf('host') === self::HOST_MODE_LOCAL;
  }

  /**
   * {@inheritdoc}
   */
  public function isCloud(): bool {
    return $this->getConf('host') === self::HOST_MODE_CLOUD;
  }

  /**
   * Delete a dir/file inside the build dir.
   *
   * @param string $localPath
   *   A local path inside the build dir.
   *
   * @return bool
   *   True on success.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function deleteInsideBuildDir(string $localPath): bool {
    $pathToDelete = $this->getConf('build-dir') . '/' . $localPath;
    exec("rm -Rf $pathToDelete", $output, $returnValue);
    if ($returnValue !== 0) {
      throw new StaticSuiteException("Unable to delete $pathToDelete:\n" . implode("\n", $output));
    }
    return TRUE;
  }

  /**
   * Copy dirs and files to a dir inside the build dir.
   *
   * @param string $source
   *   It can be external or local. If external, it must be an absolute path.
   * @param string $localDestination
   *   A path relative to release directory.
   * @param array $excludedPaths
   *   An array of paths to be excluded from the copy.
   * @param bool $createTar
   *   Flag to create a .tar file with copied data.
   *
   * @return bool
   *   True on success.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function copyDataToBuildDir(string $source, string $localDestination, array $excludedPaths, bool $createTar = FALSE): bool {
    clearstatcache(TRUE);
    if (!is_readable($source)) {
      throw new StaticSuiteException("Source $source not readable.");
    }

    if (strpos($localDestination, "/..") !== FALSE || strpos($localDestination, "../") !== FALSE) {
      throw new StaticSuiteException("Wrong parameter localDestination ($localDestination) must be a path inside " . $this->getConf('build-dir'));
    }

    $destination = $this->getConf('build-dir') . '/' . $localDestination;
    $excludeOptions = '';
    foreach ($excludedPaths as $excludedPath) {
      $excludeOptions .= "--exclude='$excludedPath' ";
    }
    clearstatcache(TRUE);
    if (is_file($source) || is_link($source)) {
      $command = "cp -a $source $destination";
    }
    else {
      $command = "rsync -av $excludeOptions $source/ $destination --delete";
    }
    exec($command, $output, $returnValue);
    if ($returnValue !== 0) {
      throw new StaticSuiteException("Unable to copy $source to $localDestination:\n" . implode("\n", $output));
    }
    // Save unique id of this data.
    file_put_contents($destination . '/' . ReleaseInterface::UNIQUE_ID_FILE_NAME, $this->uniqueId);

    if ($createTar) {
      $tarFile = '../' . $localDestination . '.tar';
      $command = 'cd ' . $destination . ' && tar -cf ' . $tarFile . ' *';
      exec($command, $output, $returnValue);
      if ($returnValue !== 0) {
        throw new StaticSuiteException("Unable to create tar file ($tarFile) from $destination:\n" . implode("\n", $output));
      }
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = [];
    $definition = $this->getPluginDefinition();
    $dependencies['module'][] = $definition['provider'];
    return $dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->configuration['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function getReleaseManager(): ReleaseManagerInterface {
    if (!$this->releaseManager->isInitialized()) {
      $this->releaseManager->init($this->getConf('run-mode-dir'));
    }
    return $this->releaseManager;
  }

  /**
   * Starts benchmark.
   */
  public function startBenchmark(): void {
    $this->timeStart = microtime(TRUE);
    $this->timeEnd = NULL;
  }

  /**
   * Ends benchmark.
   */
  public function endBenchmark(): void {
    $this->timeEnd = microtime(TRUE);
  }

  /**
   * Get elapsed time.
   *
   * @return float
   *   Elapsed time.
   */
  public function getBenchmark(): float {
    $timeEnd = $this->timeEnd ?: microtime(TRUE);
    return number_format(round($timeEnd - $this->timeStart, 3), 3);
  }

  /**
   * Get release.
   *
   * @return \Drupal\static_build\Release\Release
   *   The release we are working with.
   */
  public function getRelease(): Release {
    return $this->release;
  }

  /**
   * Get build index.
   *
   * @return int
   *   The build index.
   */
  public function getBuildIndex(): int {
    return $this->buildIndex;
  }

  /**
   * Dispatch StaticBuildEvent.
   *
   * @param string $eventName
   *   The name of the event.
   * @param array $data
   *   Data for the event.
   *
   * @return \Drupal\static_build\Event\StaticBuildEvent
   *   The event.
   */
  public function dispatchEvent(string $eventName, array $data = []): StaticBuildEvent {
    $event = new StaticBuildEvent($this);
    $event->setData($data);

    // Dispatch the event.
    $this->logMessage("EVENT DISPATCH '$eventName' TRIGGERED");
    $this->eventDispatcher->dispatch($eventName, $event);
    $this->logMessage("EVENT DISPATCH '$eventName' DONE");

    // Return the event.
    return $event;
  }

  /**
   * Validates that directory structure is correct.
   *
   * Validates everything managed by this class, inside
   * [base_dir]/[live|preview]. There is a similar method inside
   * ReleaseManagerInterface::validateDirStructure() that checks the same for
   * everything managed by ReleaseManagerInterface.
   *
   * @return array
   *   Array of translated errors if any.
   */
  protected function validateDirStructure(): array {
    $errors = [];

    $runModeDir = $this->getConf('run-mode-dir');
    if (is_dir($runModeDir)) {
      if (!is_writable($runModeDir)) {
        $errors[] = $this->t('The directory <code>%directory</code> exists but is not writable.', ['%directory' => $runModeDir]);
      }
    }
    else {
      $errors[] = $this->t('The directory <code>%directory</code> does not exist.', ['%directory' => $runModeDir]);
    }

    if ($this->isLocal()) {
      $buildDir = $this->getConf('build-dir');
      if (!is_dir($buildDir)) {
        $errors[] = $this->t('The directory <code>%directory</code> does not exist.', ['%directory' => $buildDir]);
      }
    }

    $logDir = $this->getConf('log-dir');
    if (file_exists($logDir)) {
      if (is_dir($logDir)) {
        if (!is_writable($logDir)) {
          $errors[] = $this->t('The directory <code>%directory</code> exists but is not writable.', ['%directory' => $logDir]);
        }
      }
      else {
        $errors[] = $this->t('The path <code>%directory</code> exists but is not a directory.', ['%directory' => $logDir]);
      }
    }

    $revisionsLogFile = $this->getConf('revisions-log-file');
    if (file_exists($revisionsLogFile)) {
      if (is_file($revisionsLogFile)) {
        if (!is_writable($revisionsLogFile)) {
          $errors[] = $this->t('The path <code>%path</code> exists but is not writable.', ['%path' => $revisionsLogFile]);
        }
      }
      else {
        $errors[] = $this->t('The path <code>%path</code> exists but is not a file.', ['%path' => $revisionsLogFile]);
      }
    }

    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function validateAllDirsStructure(): array {
    $builderErrors = $this->validateDirStructure();
    $releaseErrors = $this->releaseManager->validateDirStructure($this->getConf('run-mode-dir'));
    return array_merge($builderErrors, $releaseErrors);
  }

}
