<?php

namespace Drupal\static_build\Plugin;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Plugin\DependentPluginInterface;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\static_build\Release\ReleaseManagerInterface;

/**
 * Defines an interface for Static Builder plugins.
 */
interface StaticBuilderPluginInterface extends ConfigurableInterface, DependentPluginInterface, PluginInspectionInterface {

  public const HOST_MODE_LOCAL = 'local';
  public const HOST_MODE_CLOUD = 'cloud';
  public const RUN_MODE_PREVIEW = 'preview';
  public const RUN_MODE_LIVE = 'live';
  public const LOCK_MODE_PREVIEW = 'preview';
  public const LOCK_MODE_LIVE = 'live';
  public const DRUSH_ASYNC_COMMAND = 'drush static-build:build';
  public const TASK_ID = 'build';
  public const BUILD_DIR = '.build';
  public const LOG_DIR = 'log';
  public const LAST_BUILD_LOG_NAME = 'last-build.log';

  /**
   * Get unique id.
   *
   * @return string
   *   The unique id.
   */
  public function uniqueId(): string;

  /**
   * Init the build process.
   *
   * Should typically decide whether to fork an async process or run the build
   * synchronously.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function init(): bool;

  /**
   * Runs the build task.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function runBuildLoop(): bool;

  /**
   * Execute prebuild operations.
   *
   * This method should be over ridden by plugins to reflect each use case.
   */
  public function preBuild(): void;

  /**
   * Executes a build, typically a bash command running a SSG.
   *
   * This method should be over ridden by plugins to reflect each use case.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function build(): void;

  /**
   * Execute post build operations.
   *
   * This method should be over ridden by plugins to reflect each use case.
   */
  public function postBuild(): void;

  /**
   * Check if we really need to build the site again.
   *
   * It honors "last locked executed unique id" from static_export, since that's
   * the value that holds when data was last changed. If current release differs
   * from that value, we must build the site again, except when changed files
   * do not match any of the regexps defined on setBuildTriggerRegexps()
   *
   * @return bool
   *   True if we must trigger a build.
   */
  public function mustBuild(): bool;

  /**
   * Get an array of paths to exclude from data copy.
   *
   * This method should be over ridden by plugins to reflect each use case.
   *
   * @return array
   *   Array of paths.
   */
  public function getDataDirExcludedPaths(): array;

  /**
   * Get the configured release manager.
   *
   * @return \Drupal\static_build\Release\ReleaseManagerInterface
   *   The current release manager.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function getReleaseManager(): ReleaseManagerInterface;

  /**
   * Get a conf value.
   *
   * @param string $key
   *   The conf key to be returned.
   *
   * @return mixed|null
   *   The conf value
   */
  public function getConf($key);

  /**
   * Tells whether a builder's host is local or not.
   *
   * @return bool
   *   TRUE if it's local or FALSE otherwise
   */
  public function isLocal(): bool;

  /**
   * Tells whether a builder's host is cloud or not.
   *
   * @return bool
   *   TRUE if it's cloud or FALSE otherwise
   */
  public function isCloud(): bool;

  /**
   * Get the id of the task this builder is running.
   *
   * All builders execute a task that must be identified. It's id is usually
   * "build-[PLUGIN_ID]".
   *
   * @return string
   *   The task id
   */
  public function getTaskId(): string;

  /**
   * Logs a message.
   *
   * Don't throw any error. It's essential to ensure proper recovery on
   * try-catch blocks.
   *
   * @param string $message
   *   Message to log.
   * @param bool $mustFormat
   *   Optional. Boolean to indicate that message must be formatted.
   */
  public function logMessage(string $message, bool $mustFormat = TRUE);

  /**
   * Validates that all directory structures are correct.
   *
   * Validates both directories/files managed by this class and
   * ReleaseManagerInterface.
   *
   * @return array
   *   Array of translated errors if any.
   */
  public function validateAllDirsStructure(): array;

}
