<?php

namespace Drupal\static_build\Release\Task;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\static_build\Release\Release;
use Drupal\static_build\Release\ReleaseManagerInterface;
use Drupal\static_suite\Utility\UniqueIdHelperInterface;
use Throwable;

/**
 * A class to control tasks inside releases.
 */
class TaskSupervisor implements TaskSupervisorInterface {

  /**
   * Unique ID helper.
   *
   * @var \Drupal\static_suite\Utility\UniqueIdHelperInterface
   */
  protected $uniqueIdHelper;

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The release manager.
   *
   * @var \Drupal\static_build\Release\ReleaseManagerInterface
   */
  protected $releaseManager;

  /**
   * Task supervisor constructor.
   *
   * Although this supervisor can be directly accessed and used, it should be
   * accessed through a plugin's getReleaseManager()->getTaskSupervisor()
   * method. This way, release manager is already configured and initialized,
   * honoring that plugin's configuration. This is also essential to allow
   * different plugins with different base directories (a Work In Progress not
   * already finished).
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal config factory.
   * @param \Drupal\static_suite\Utility\UniqueIdHelperInterface $unique_id_helper
   *   Static Export utils.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    UniqueIdHelperInterface $unique_id_helper
  ) {
    $this->configFactory = $config_factory;
    $this->uniqueIdHelper = $unique_id_helper;
  }

  /**
   * Set the release manager.
   *
   * @param \Drupal\static_build\Release\ReleaseManagerInterface $releaseManager
   *   The release manager.
   */
  public function setReleaseManager(ReleaseManagerInterface $releaseManager): void {
    $this->releaseManager = $releaseManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getLastDoneRelease(string $taskId = NULL): ?Release {
    $allReleases = $this->releaseManager->getAllReleases();
    foreach ($allReleases as $release) {
      if ($release->task($taskId)->isDone()) {
        return $release;
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getMedianTaskTime(string $taskId = NULL): int {
    $numberOfReleasesToKeep = $this->configFactory->get('static_build.settings')->get('number_of_releases_to_keep');
    /* @var $releasesToCalculateMedian Release[] */
    $releasesToCalculateMedian = array_slice(
        $this->releaseManager->getAllReleases(),
        0,
        $numberOfReleasesToKeep
      );

    $numberOfValidReleases = 0;
    if ($releasesToCalculateMedian) {
      // Get a estimated time from last N releases to compare.
      $timeSum = 0;
      foreach ($releasesToCalculateMedian as $release) {
        if ($release->task($taskId)->isDone()) {
          $timeSum += $release->task($taskId)->getProcessBenchmark();
          $numberOfValidReleases++;
        }
      }
      if ($numberOfValidReleases) {
        $medianTime = (int) round($timeSum / $numberOfValidReleases, 0);
      }
      else {
        // This happens when no release is done.
        $medianTime = 0;
      }

    }
    else {
      // This happens when no release is present.
      $medianTime = 0;
    }

    return $medianTime;
  }

  /**
   * {@inheritdoc}
   */
  public function getTaskPercentage(string $uniqueId, string $taskId = NULL): int {
    $percentage = 0;
    $ongoingRelease = $this->releaseManager->create($uniqueId);
    $ongoingReleaseTask = $ongoingRelease->task($taskId);

    if ($ongoingRelease) {
      if ($ongoingReleaseTask->isDone()) {
        $percentage = 100;
      }
      elseif ($ongoingReleaseTask->isFailed()) {
        $percentage = 0;
      }
      else {
        $medianTaskTime = $this->getMedianTaskTime($taskId);
        $elapsedSeconds = $ongoingReleaseTask->getProcessElapsedSeconds();
        $percentage = $medianTaskTime ? (int) round(($elapsedSeconds / $medianTaskTime) * 100, 0) : 0;
        if ($percentage < 0) {
          $percentage = 0;
        }
        elseif ($percentage > 99) {
          // Don't show 100% because we don't really know when it's going to
          // finish.
          $percentage = 99;
        }
      }
    }

    return $percentage;
  }

  /**
   * {@inheritdoc}
   */
  public function getRemainingSeconds(string $uniqueId, string $taskId = NULL): int {
    $remainingSeconds = 0;
    $ongoingRelease = $this->releaseManager->create($uniqueId);
    $ongoingReleaseTask = $ongoingRelease->task($taskId);

    if ($ongoingRelease) {
      if ($ongoingReleaseTask->isDone() || $ongoingReleaseTask->isFailed()) {
        $remainingSeconds = 0;
      }
      else {
        // This could be a negative number. Let's keep it as-is, to denote a
        // task is taking more time than expected.
        $remainingSeconds = $this->getMedianTaskTime($taskId) - $ongoingReleaseTask->getProcessElapsedSeconds();
      }
    }

    return $remainingSeconds;
  }

  /**
   * {@inheritdoc}
   */
  public function getOngoingRelease(string $taskId = NULL): ?Release {
    $newestRelease = $this->releaseManager->getNewestRelease();

    // Get ongoing release.
    if (
      $newestRelease &&
      $this->releaseManager->exists($newestRelease->uniqueId()) &&
      !$newestRelease->task($taskId)->isDone() &&
      !$newestRelease->task($taskId)->isFailed()
    ) {
      return $newestRelease;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getOngoingReleaseTaskData(string $taskId = NULL): array {
    $data = self::DEFAULT_TASK_DATA;
    $onGoingRelease = $this->getOngoingRelease($taskId);
    if ($onGoingRelease) {
      $data = $this->getReleaseTaskData($onGoingRelease->uniqueId(), $taskId);
    }
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function getReleaseTaskData(string $uniqueId, string $taskId = NULL): array {
    $data = self::DEFAULT_TASK_DATA;
    $elapsedSeconds = 0;
    $remainingSeconds = 0;
    $percentage = 0;
    $release = $this->releaseManager->create($uniqueId);
    $releaseTask = $release->task($taskId);
    if ($this->releaseManager->exists($uniqueId)) {
      $elapsedSeconds = $releaseTask->getProcessElapsedSeconds();
      $remainingSeconds = $this->getRemainingSeconds($uniqueId, $taskId);
      $percentage = $this->getTaskPercentage($uniqueId, $taskId);
    }

    $uniqueIdTimestamp = 0;
    try {
      $uniqueIdDate = $this->uniqueIdHelper->getDateFromUniqueId($uniqueId);
      $uniqueIdTimestamp = $uniqueIdDate->format("U.u");
    }
    catch (Throwable $e) {
      // Do nothing.
    }

    $data['unique-id'] = $uniqueId;
    $data['elapsed-seconds'] = $elapsedSeconds;
    $data['remaining-seconds'] = $remainingSeconds;
    $data['percentage'] = $percentage;
    $data['unique-id-timestamp'] = $uniqueIdTimestamp;

    return $data;
  }

}
