<?php

namespace Drupal\static_build\Release\Task;

use Drupal\static_build\Release\Release;
use Drupal\static_build\Release\ReleaseManagerInterface;

/**
 * An interface for task supervisors.
 */
interface TaskSupervisorInterface {
  public const DEFAULT_TASK_DATA = [
    'unique-id' => NULL,
    'elapsed-seconds' => 0,
    'remaining-seconds' => 0,
    'percentage' => 0,
  ];

  /**
   * Set the release manager.
   *
   * @param \Drupal\static_build\Release\ReleaseManagerInterface $releaseManager
   *   The release manager.
   */
  public function setReleaseManager(ReleaseManagerInterface $releaseManager): void;

  /**
   * Get last done release.
   *
   * @param string $taskId
   *   Task id to check.
   *
   * @return \Drupal\static_build\Release\Release|null
   *   Last done release or null if nothing found
   */
  public function getLastDoneRelease(string $taskId): ?Release;

  /**
   * Get median task time.
   *
   * @param string $taskId
   *   Task id to check.
   *
   * @return int
   *   Median task time in seconds
   */
  public function getMedianTaskTime(string $taskId): int;

  /**
   * Get percentage of a release task.
   *
   * @param string $uniqueId
   *   A unique Id identifying a release.
   * @param string $taskId
   *   Task id to check.
   *
   * @return int
   *   Task percentage.
   */
  public function getTaskPercentage(string $uniqueId, string $taskId): int;

  /**
   * Get estimated number of seconds remaining for a release task to finish.
   *
   * @param string $uniqueId
   *   A unique Id identifying a release.
   * @param string $taskId
   *   Task id to check.
   *
   * @return int
   *   Remaining seconds.
   */
  public function getRemainingSeconds(string $uniqueId, string $taskId): int;

  /**
   * Get the release with a running task..
   *
   * @param string $taskId
   *   Task id to check.
   *
   * @return \Drupal\static_build\Release\Release|null
   *   The release with a running task or null if nothing found.
   */
  public function getOngoingRelease(string $taskId): ?Release;

  /**
   * Get data from a task inside a release.
   *
   * @param string $taskId
   *   Task id to check.
   *
   * @return array
   *   An array with data from the release.
   */
  public function getOngoingReleaseTaskData(string $taskId): array;

  /**
   * Get task data from a release.
   *
   * @param string $uniqueId
   *   Release's unique id.
   * @param string $taskId
   *   Task id to check.
   *
   * @return array
   *   An array with data from the release.
   */
  public function getReleaseTaskData(string $uniqueId, string $taskId): array;

}
