<?php

namespace Drupal\static_deploy\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface;
use Drupal\static_build\Release\ReleaseManagerInterface;
use Drupal\static_deploy\Event\StaticDeployEvent;
use Drupal\static_deploy\Event\StaticDeployEvents;
use Drupal\static_build\Release\ReleaseManager;
use Drupal\static_suite\Lock\LockBackendInterface;
use Drupal\static_suite\StaticSuiteException;
use Drupal\static_suite\StaticSuiteUserException;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Throwable;

/**
 * Base class for Static Builder plugin plugins.
 */
abstract class StaticDeployerPluginBase extends PluginBase implements StaticDeployerPluginInterface, ContainerFactoryPluginInterface {
  use LoggerChannelTrait;

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Drupal file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Release Manager.
   *
   * @var \Drupal\static_build\Release\ReleaseManagerInterface
   */
  protected $releaseManager;

  /**
   * The lock system from Static Suite.
   *
   * @var \Drupal\Core\Lock\LockBackendInterface
   */
  protected $lock;

  /**
   * Static Export utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * An array of messages to be logged.
   *
   * @var array
   */
  protected $logMessagesStack;

  /**
   * A flag to enable logging at a later moment, when all check are passed.
   *
   * @var bool
   */
  protected $startLogging = FALSE;

  /**
   * Current release.
   *
   * @var \Drupal\static_build\Release\ReleaseInterface
   */
  protected $currentRelease;

  /**
   * Current release task.
   *
   * @var \Drupal\static_build\Release\Task\Task
   */
  protected $releaseTask;

  /**
   * Time start.
   *
   * @var float
   */
  protected $timeStart;

  /**
   * Time end.
   *
   * @var float
   */
  protected $timeEnd;

  /**
   * The static builder manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface
   */
  protected $staticBuilderPluginManager;

  /**
   * The static builder that this deployer is using.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderPluginInterface
   */
  protected $staticBuilder;

  /**
   * {@inheritdoc}
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config service.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   Drupal file system service.
   * @param \Drupal\static_suite\Lock\LockBackendInterface $lock
   *   The lock system from Static Suite.
   * @param \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface $static_builder_manager
   *   The static builder plugin manager.
   * @param \Drupal\static_build\Release\ReleaseManager $releaseManager
   *   The release manager service.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $static_suite_utils
   *   Static Export utils.
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $configFactory,
    EventDispatcherInterface $eventDispatcher,
    FileSystemInterface $fileSystem,
    LockBackendInterface $lock,
    StaticBuilderPluginManagerInterface $static_builder_manager,
    ReleaseManager $releaseManager,
    StaticSuiteUtilsInterface $static_suite_utils
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $configFactory;
    $this->eventDispatcher = $eventDispatcher;
    $this->fileSystem = $fileSystem;
    $this->lock = $lock;
    $this->staticBuilderPluginManager = $static_builder_manager;
    $this->releaseManager = $releaseManager;
    $this->staticSuiteUtils = $static_suite_utils;
    $this->setConfiguration($configuration);
    $this->setDerivedConfiguration();
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public static function create(ContainerInterface $container,
                                array $configuration,
                                $plugin_id,
                                $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get("config.factory"),
      $container->get("event_dispatcher"),
      $container->get("file_system"),
      $container->get("static_suite.lock"),
      $container->get("plugin.manager.static_builder"),
      $container->get("static_build.release_manager"),
      $container->get("static_suite.utils")
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getTaskId(): string {
    return self::TASK_ID . '-' . $this->getConf('id');
  }

  /**
   * {@inheritdoc}
   */
  public function init() {
    // Fork an async process only if not in a cli and async option is true.
    if (!$this->staticSuiteUtils->isRunningOnCli() && $this->getConf('sync') === FALSE) {
      // Define a log file so we can get info about the forking process.
      $forkLog = $this->fileSystem->getTempDirectory() . '/static_deploy_fork.' . $this->getConf('id');
      $forkLog .= $this->getConf('drush-options') ? '.' . md5($this->getConf('drush-options')) : '';
      $forkLog .= '.log';

      // Run async command.
      $asyncCommands = array_merge(
        $this->getEnvironmentVariables(),
        [
          'cd ' . DRUPAL_ROOT,
          self::DRUSH_ASYNC_COMMAND . ' ' . implode(' ', $this->getConf('drush-options')) . ' > ' . $forkLog . ' 2>&1 &',
        ]
      );

      $this->dispatchEvent(StaticDeployEvents::ASYNC_PROCESS_FORK_START);
      exec(implode(' && ', $asyncCommands), $output, $returnValue);
      $this->dispatchEvent(StaticDeployEvents::ASYNC_PROCESS_FORK_END);
      if ($returnValue !== 0) {
        throw new StaticSuiteException("Unable to fork deploy process:\n" . implode("\n", $output));
      }

      return $returnValue;
    }
    // Otherwise, run it synchronously.
    return $this->run();
  }

  /**
   * {@inheritdoc}
   */
  public function run() {
    $deployLockName = $this->getConf('run-mode-dir') . '--deploy';
    try {
      $this->releaseManager->init($this->getConf('run-mode-dir'));

      $this->startBenchmark();

      $this->logMessage('**********************************************************');
      $this->logMessage('DEPLOY RUN STARTS');
      $this->logMessage('**********************************************************');

      if (!$this->lock->acquire($deployLockName, $this->configFactory->get('static_deploy.settings')->get('semaphore_timeout'))) {
        $this->logMessage('Another deploy process is running. Aborting this one.');
        $this->logMessage("**********************************************************");
        $this->logMessage("DEPLOY RUN ABORTED");
        $this->logMessage("**********************************************************\n");
        return FALSE;
      }

      $this->dispatchEvent(StaticDeployEvents::START);

      $this->logMessage('No other deploy process running. Go ahead with this one.');
      $this->logMessage('Deploy lock acquired.');

      // Discover current release.
      $this->currentRelease = $this->releaseManager->getCurrentRelease();
      $this->releaseTask = $this->currentRelease->task($this->getTaskId());
      $this->logMessage('Discovered current release: ' . $this->currentRelease->getDir());

      // Check that current release hasn't been deployed.
      if (!$this->getConf('force') && $this->releaseTask->isDone()) {
        // We simply return without throwing. This is not an error.
        $this->logMessage('Release ' . $this->currentRelease->getDir() . ' is already deployed. Aborting...');
        return FALSE;
      }

      // Check that current release is built.
      if (!$this->currentRelease->task($this->staticBuilder->getTaskId())->isDone()) {
        throw new StaticSuiteException('Release ' . $this->currentRelease->getDir() . " is not done. This a major issue that shouldn't happen. Aborting...");
      }

      // To avoid unnecessary logging, we only log when all checks are OK
      // and a real deploy is triggered.
      $this->startLogging = TRUE;

      // Set current release as started.
      $this->logMessage('Release marked as started.');
      $this->releaseTask->setStarted();

      // Execute deploy operation from plugin.
      $this->dispatchEvent(StaticDeployEvents::DEPLOY_START);
      $this->deploy();
      $this->dispatchEvent(StaticDeployEvents::DEPLOY_END);

      // Set current release as deployed.
      $this->logMessage('Release marked as deployed.');
      $this->releaseTask->setDone();

      // RELEASE DEPLOY LOCK.
      $this->lock->release($deployLockName);
      $this->logMessage('Deploy lock released.');

      $this->getBenchmark();
      $this->logMessage('Elapsed time: ' . $this->getBenchmark() . 'sec.');

      $this->logMessage('**********************************************************');
      $this->logMessage('DEPLOY RUN ENDS');
      $this->logMessage('**********************************************************');

      $this->dispatchEvent(StaticDeployEvents::END);

      return TRUE;
    }
    catch (Throwable $e) {
      // This is Throwable to capture any kind of exception and ensure
      // that any pending work is done.
      $this->logMessage('Exception thrown. Cleaning up...');
      $this->logMessage($e->getMessage() . "\n" . $e->getTraceAsString());

      $this->dispatchEvent(StaticDeployEvents::ROLLBACK_START);
      $this->rollback();
      $this->dispatchEvent(StaticDeployEvents::ROLLBACK_END);

      // If this exception comes from Static Build, log an error.
      if ($e instanceof StaticSuiteException) {
        $message = $e->getMessage() . ' Please, review log files.';
        $this->getLogger('static_deploy')->error($message);
      }

      // Mark release as failed.
      if ($this->releaseTask && $this->releaseTask->isStarted()) {
        $this->releaseTask->setFailed();
      }

      // Release any pending lock.
      $this->lock->release($deployLockName);
      $this->logMessage('Deploy lock released.');

      throw new StaticSuiteException($e);
    }
    finally {
      $this->lock->releaseAll();
    }
  }

  /**
   * Logs a message.
   *
   * Don't throw any error. It's essential to ensure proper recovery on
   * try-catch blocks.
   */
  protected function logMessage(string $message) {
    try {
      if (!empty($message)) {
        $timestamp = $this->staticSuiteUtils->getFormattedMicroDate('Y-m-d H:i:s.u');
        $uniqueIdStamp = $this->currentRelease ? '[' . $this->currentRelease->uniqueId() . '] ' : '';
        $formattedMessage = getmypid() . ' [' . $timestamp . '] ' . $uniqueIdStamp . '[' . $this->getConf('id') . '] ' . $message;

        // To avoid unnecessary logging, we only log when all checks are OK
        // and a real deploy happens.
        $logFilePath = $this->releaseTask ? $this->releaseTask->getLogFilePath() : NULL;
        if ($this->startLogging && $logFilePath && is_writable(dirname($logFilePath))) {
          $messageToLog = $formattedMessage;
          if (count($this->logMessagesStack)) {
            $messageToLog = implode("\n", $this->logMessagesStack) . "\n" . $formattedMessage;
            unset($this->logMessagesStack);
          }
          @file_put_contents($logFilePath, "$messageToLog\n", FILE_APPEND | LOCK_EX);
        }
        else {
          $this->logMessagesStack[] = $formattedMessage;
        }

        if ($this->getConf('console-output')) {
          $this->getConf('console-output')->writeln($formattedMessage);
        }
      }
    }
    catch (Throwable $e) {
      echo $e->getMessage();
      // Do nothing.
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function setConfiguration(array $configuration) {
    // Check that builder-id option is present.
    if (empty($configuration['builder-id'])) {
      throw new StaticSuiteUserException('Builder ID is empty.');
    }

    if (!array_key_exists($configuration['builder-id'], $this->staticBuilderPluginManager->getLocalDefinitions())) {
      throw new StaticSuiteUserException('Builder "' . $configuration['builder-id'] . '" is not a builder of type "' . StaticBuilderPluginInterface::HOST_MODE_LOCAL . '"');
    }

    // Merge config and get a new one.
    $mergedConf = NestedArray::mergeDeep(
      $this->baseConfigurationDefaults($configuration['builder-id']),
      $this->defaultConfiguration(),
      $configuration
    );

    // Finally, set the config.
    $this->configuration = $mergedConf;
  }

  /**
   * Set other config values derived from current ones.
   *
   * This method is executed once base configuration and plugin (AWS, Azure)
   * configuration have been applied. This way, different plugins can define
   * different base directories (base_dir) and be run in parallel.
   */
  protected function setDerivedConfiguration() {
    $this->setConf('run-mode-dir', $this->getConf('base-dir') . '/' . StaticBuilderPluginInterface::RUN_MODE_LIVE);
  }

  /**
   * Returns generic default configuration for static builder plugins.
   *
   * This must be overridden by any plugin using defaultConfiguration()
   * and setConfiguration().
   *
   * @param string $builderId
   *   Static builder ID.
   *
   * @return array
   *   An associative array with the default configuration.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  protected function baseConfigurationDefaults(string $builderId): array {
    $config = $this->configFactory->get('static_deploy.settings');
    $this->staticBuilder = $this->staticBuilderPluginManager->getInstance([
      'plugin_id' => $builderId,
      'configuration' => ['run-mode' => StaticBuilderPluginInterface::RUN_MODE_LIVE],
    ]);

    $baseConf = [
      'id' => $this->getPluginId(),
      'label' => $this->pluginDefinition['label'],
      'description' => $this->pluginDefinition['description'],
      'base-dir' => $this->staticBuilder->getConf('base-dir'),
      'drush-env' => $config->get('drush_env'),
      'drush-options' => $config->get('drush_options'),
      'deploy-request-policy' => $config->get('deploy_request_policy'),
      'sync' => empty($config->get('sync')) ? FALSE : TRUE,
    ];

    return $baseConf;
  }

  /**
   * Set a conf value.
   *
   * @param string $key
   *   Thd config key.
   * @param string $value
   *   Thd config value.
   */
  protected function setConf(string $key, string $value) {
    $this->configuration[$key] = $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConf(string $key) {
    return $this->configuration[$key] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = [];
    $definition = $this->getPluginDefinition();
    $dependencies['module'][] = $definition['provider'];
    return $dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    $label = '';
    if (!empty($this->configuration['label'])) {
      $label = $this->configuration['label'];
    }
    return $label;
  }

  /**
   * {@inheritdoc}
   */
  public function getReleaseManager(): ReleaseManagerInterface {
    if (!$this->releaseManager->isInitialized()) {
      $this->releaseManager->init($this->getConf('run-mode-dir'));
    }
    return $this->releaseManager;
  }

  /**
   * Starts benchmark.
   */
  protected function startBenchmark() {
    $this->timeStart = microtime(TRUE);
    $this->timeEnd = NULL;
  }

  /**
   * Ends benchmark.
   */
  protected function endBenchmark() {
    $this->timeEnd = microtime(TRUE);
  }

  /**
   * Get elapsed time.
   *
   * @return float
   *   Elapsed time.
   */
  protected function getBenchmark(): float {
    $timeEnd = $this->timeEnd ? $this->timeEnd : microtime(TRUE);
    return number_format(round($timeEnd - $this->timeStart, 3), 3);
  }

  /**
   * Dispatch StaticDeployEvent.
   *
   * @param string $eventName
   *   The name of the event.
   * @param array $data
   *   Data for the event.
   *
   * @return \Drupal\static_deploy\Event\StaticDeployEvent
   *   The event.
   */
  protected function dispatchEvent(string $eventName, array $data = []): StaticDeployEvent {
    $event = new StaticDeployEvent($this);
    $event->setData($data);

    // Dispatch the event.
    $this->logMessage("EVENT DISPATCH '$eventName' TRIGGERED");
    $this->eventDispatcher->dispatch($eventName, $event);
    $this->logMessage("EVENT DISPATCH '$eventName' DONE");

    // Return the event.
    return $event;
  }

  /**
   * Get environment variables for forked process.
   *
   * @todo Find a way to avoid duplicating this code in StaticBuilderBase
   *
   * @return array
   *   Array with environment commands.
   */
  protected function getEnvironmentVariables(): array {
    $env = $this->getConf('env');
    if (!is_array($env)) {
      $env = [];
    }

    // Search for $HOME and $PATH inside $env.
    $isHomeDefined = FALSE;
    $isPathDefined = FALSE;
    foreach ($env as $envLine) {
      if (preg_match("/^\s*export\s*HOME=/", $envLine)) {
        $isHomeDefined = TRUE;
      }
      elseif (preg_match("/^\s*export\s*PATH=/", $envLine)) {
        $isPathDefined = TRUE;
      }
    }

    // Set default $HOME and $PATH if not defined in $env.
    if (!$isHomeDefined) {
      $env[] = 'export HOME=' . $this->staticSuiteUtils->discoverDefaultHome();
    }
    if (!$isPathDefined) {
      $env[] = 'export PATH=/usr/local/bin:/usr/bin:/bin';
    }

    $environmentCommands = [];
    if (is_array($env)) {
      foreach ($env as $envLine) {
        // Ensure only lines starting with "export" are executed.
        if (preg_match("/^\s*export\s+/", $envLine)) {
          $environmentCommands[] = $envLine;
        }
      }
    }

    return $environmentCommands;
  }

}
