<?php

namespace Drupal\static_export\Commands;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\static_export\File\FileCollectionFormatter;
use Drupal\static_export\Exporter\Type\Locale\LocaleExporterPluginManagerInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Drush\Commands\DrushCommands;
use Throwable;

/**
 * A Drush command file to export locale data.
 */
class LocaleCommands extends DrushCommands {

  /**
   * Language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Locale exporter manager.
   *
   * @var \Drupal\static_export\Exporter\Type\Locale\LocaleExporterPluginManagerInterface
   */
  protected $localeExporterManager;

  /**
   * File collection formatter.
   *
   * @var \Drupal\static_export\File\FileCollectionFormatter
   */
  protected $fileCollectionFormatter;

  /**
   * StaticExportCommands constructor.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager.
   * @param \Drupal\static_export\Exporter\Type\Locale\LocaleExporterPluginManagerInterface $localeExporterManager
   *   Locale exporter manager.
   * @param \Drupal\static_export\File\FileCollectionFormatter $file_collection_formatter
   *   File collection formatter.
   */
  public function __construct(LanguageManagerInterface $language_manager, LocaleExporterPluginManagerInterface $localeExporterManager, FileCollectionFormatter $file_collection_formatter) {
    parent::__construct();
    $this->languageManager = $language_manager;
    $this->localeExporterManager = $localeExporterManager;
    $this->fileCollectionFormatter = $file_collection_formatter;
  }

  /**
   * Exports localized strings.
   *
   * @param string|null $languageCode
   *   The language code to export. Optional. If none provided, all languages
   *   are exported.
   * @param array $execOptions
   *   An associative array of options whose values come
   *   from cli, aliases, config, etc.
   *
   * @command static-export:export-locale
   *
   * @option standalone Flag to set standalone mode
   * @usage drush static-export:export-locale pt-br
   *   Exports locales.
   * @aliases sel, static-export-locale
   *
   * @static_export Annotation for drush hooks.
   * @static_export_data_dir_write Annotation for commands that write data.
   */
  public function exportLocale(
    $languageCode = NULL,
    array $execOptions = [
      'standalone' => FALSE,
      'log-to-file' => TRUE,
      'lock' => TRUE,
      'build' => FALSE,
    ]
  ): void {
    $timeStart = microtime(TRUE);

    $languageCodesToExport = [];
    if ($languageCode) {
      $languageCodesToExport[] = $languageCode;
    }
    else {
      foreach ($this->languageManager->getLanguages() as $language) {
        $languageCodesToExport[] = $language->getId();
      }
    }

    try {
      $total = count($languageCodesToExport);
      $localeExporter = $this->localeExporterManager->getDefaultInstance();
      foreach ($languageCodesToExport as $delta => $langCode) {
        $localeExporter->setMustBuild($execOptions['build']);
        $fileCollectionGroup = $localeExporter->export(
          ['language' => $langCode],
          $execOptions['standalone'],
          $execOptions['log-to-file'],
          $execOptions['lock']
        );
        foreach ($fileCollectionGroup->getFileCollections() as $fileCollection) {
          $this->fileCollectionFormatter->setFileCollection($fileCollection);
          $this->output()
            ->writeln($this->fileCollectionFormatter->getTextLines($delta + 1, $total));
        }
      }
      $this->output()->writeln("TOTAL TIME: " . (round(microtime(TRUE) - $timeStart, 3)) . " secs.");
      // $this->entityExporter->logger()->info($message);
    }
    catch (StaticSuiteUserException | PluginException $e) {
      $this->logger()->error($e->getMessage());
    }
    catch (Throwable $e) {
      $this->logger()->error($e);
    }
  }

}
