<?php

namespace Drupal\static_export\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Url;
use Drupal\static_export\Exporter\Data\Resolver\DataResolverPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a listing of data resolvers.
 */
class DataResolversList extends ControllerBase {

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The data resolver manager.
   *
   * @var \Drupal\static_export\Exporter\Data\Resolver\DataResolverPluginManagerInterface
   */
  protected $dataResolverManager;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The route provider.
   * @param \Drupal\static_export\Exporter\Data\Resolver\DataResolverPluginManagerInterface $dataResolverManager
   *   The resolver manager.
   */
  public function __construct(RouteProviderInterface $route_provider, DataResolverPluginManagerInterface $dataResolverManager) {
    $this->routeProvider = $route_provider;
    $this->dataResolverManager = $dataResolverManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('router.route_provider'),
      $container->get('plugin.manager.static_data_resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function listDataResolvers() {
    $tableHeader = [
      'id' => $this->t('ID'),
      'name' => $this->t('Name'),
      'description' => $this->t('Description'),
      'configuration' => $this->t('Configuration'),
    ];
    $definitions = $this->dataResolverManager->getDefinitions();
    $tableRows = NULL;
    foreach ($definitions as $pluginId => $pluginDefinition) {
      $row = [
        'id' => $pluginId,
        'name' => $pluginDefinition['label'],
        'description' => $pluginDefinition['description'],
      ];
      if (count($this->routeProvider->getRoutesByNames(['static_export_' . $pluginId . '_data_resolver.settings'])) === 1) {
        $row['configuration']['data'] = [
          '#title' => $this->t('Edit configuration'),
          '#type' => 'link',
          '#url' => Url::fromRoute('static_export_' . $pluginId . '_data_resolver.settings'),
        ];
      }
      else {
        $row['configuration'] = $this->t('No configuration available');
      }
      $tableRows[$pluginId] = $row;
    }

    return [
      '#type' => 'table',
      '#header' => $tableHeader,
      '#rows' => $tableRows,
      '#prefix' => '<p>' . $this->t(
          'Each entity can be exported using a different data resolver. All available data resolvers are listed below. Data resolvers are extensible plugins, so if no one matches your needs, you can define your own one by creating a plugin with a "@annotation" annotation.', ['@annotation' => '@StaticDataResolver']
      ) . '</p>',
      '#suffix' => count($tableRows) === 0 ? '<p>' . $this->t('No data resolver found') . '</p>' : NULL,
    ];

  }

}
