<?php

namespace Drupal\static_export\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Url;
use Drupal\static_export\Exporter\Output\Formatter\OutputFormatterPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a listing of output formatters.
 */
class OutputFormattersList extends ControllerBase {

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The output formatter manager.
   *
   * @var \Drupal\static_export\Exporter\Output\Formatter\OutputFormatterPluginManagerInterface
   */
  protected $outputFormatterManager;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The route provider.
   * @param \Drupal\static_export\Exporter\Output\Formatter\OutputFormatterPluginManagerInterface $outputFormatterManager
   *   The output formatter manager.
   */
  public function __construct(RouteProviderInterface $route_provider, OutputFormatterPluginManagerInterface $outputFormatterManager) {
    $this->routeProvider = $route_provider;
    $this->outputFormatterManager = $outputFormatterManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('router.route_provider'),
      $container->get('plugin.manager.static_output_formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function listFormatters() {
    $tableHeader = [
      'id' => $this->t('ID'),
      'name' => $this->t('Name'),
      'description' => $this->t('Description'),
      'configuration' => $this->t('Configuration'),
    ];
    $definitions = $this->outputFormatterManager->getDefinitions();
    $tableRows = NULL;
    foreach ($definitions as $pluginId => $pluginDefinition) {
      $row = [
        'id' => $pluginId,
        'name' => $pluginDefinition['label'],
        'description' => $pluginDefinition['description'],
      ];
      if (count($this->routeProvider->getRoutesByNames(['static_export_output_formatter_' . $pluginId . '.settings'])) === 1) {
        $row['configuration']['data'] = [
          '#title' => $this->t('Edit configuration'),
          '#type' => 'link',
          '#url' => Url::fromRoute('static_export_output_formatter_' . $pluginId . '.settings'),
        ];
      }
      else {
        $row['configuration'] = $this->t('No configuration available');
      }
      $tableRows[$pluginId] = $row;
    }

    return [
      '#type' => 'table',
      '#header' => $tableHeader,
      '#rows' => $tableRows,
      '#prefix' => '<p>' . $this->t(
          'Exportable entities, configurations and locales can be exported using several output formatters. All available formatters are listed below. Formatters are extensible plugins, so if no one matches your needs, you can define your own one by creating a plugin with a "@annotation" annotation.', ['@annotation' => '@StaticOutputFormatter']
      ) . '</p>',
      '#suffix' => count($tableRows) === 0 ? '<p>' . $this->t('No resolver found') . '</p>' : NULL,
    ];

  }

}
