<?php

namespace Drupal\static_export\EventSubscriber;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\language\Config\LanguageConfigOverrideCrudEvent;
use Drupal\language\Config\LanguageConfigOverrideEvents;
use Drupal\static_export\Exporter\Type\Custom\CustomExporterPluginManagerInterface;
use Drupal\static_export\Messenger\Messenger;
use Drupal\static_suite\StaticSuiteUserException;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Throwable;

/**
 * Event subscriber for custom language config exporter.
 */
class LanguageConfigEventSubscriber implements EventSubscriberInterface {

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Custom exporter manager.
   *
   * @var \Drupal\static_export\Exporter\Type\Custom\CustomExporterPluginManagerInterface
   */
  protected $customExporterPluginManager;

  /**
   * Messenger service.
   *
   * @var \Drupal\static_export\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Static Suite Utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * The exporter that will handle the event.
   *
   * @var \Drupal\static_export\Exporter\ExporterPluginInterface
   */
  protected $exporter;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config factory.
   * @param \Drupal\static_export\Exporter\Type\Custom\CustomExporterPluginManagerInterface $customExporterPluginManager
   *   Custom exporter manager.
   * @param \Drupal\static_export\Messenger\Messenger $messenger
   *   Messenger service.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $staticSuiteUtils
   *   Static Suite Utils.
   */
  public function __construct(ConfigFactoryInterface $configFactory, CustomExporterPluginManagerInterface $customExporterPluginManager, Messenger $messenger, StaticSuiteUtilsInterface $staticSuiteUtils) {
    $this->configFactory = $configFactory;
    $this->customExporterPluginManager = $customExporterPluginManager;
    $this->messenger = $messenger;
    $this->staticSuiteUtils = $staticSuiteUtils;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ConfigEvents::SAVE][] = ['configCrudHandler'];
    $events[ConfigEvents::RENAME][] = ['configCrudHandler'];
    $events[ConfigEvents::DELETE][] = ['configCrudHandler'];
    $events[LanguageConfigOverrideEvents::SAVE_OVERRIDE][] = ['configSaveOverrideHandler'];
    $events[LanguageConfigOverrideEvents::DELETE_OVERRIDE][] = ['configSaveOverrideHandler'];
    return $events;
  }

  /**
   * Reacts to a CRUD event on language.negotiation.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The configuration event.
   *
   * @return bool
   *   TRUE if everything is OK.
   */
  public function configCrudHandler(ConfigCrudEvent $event): bool {
    $configName = $event->getConfig()->getName();
    if (!self::affectsLanguages($configName)) {
      return FALSE;
    }
    return $this->export();
  }

  /**
   * Reacts to a CRUD override event on language.negotiation.
   *
   * @param \Drupal\language\Config\LanguageConfigOverrideCrudEvent $event
   *   The configuration event.
   *
   * @return bool
   *   TRUE if everything is OK.
   */
  public function configSaveOverrideHandler(LanguageConfigOverrideCrudEvent $event): bool {
    $configName = $event->getLanguageConfigOverride()->getName();
    if (!self::affectsLanguages($configName)) {
      return FALSE;
    }
    return $this->export();
  }

  /**
   * Exports language configuration data.
   *
   * @return bool
   *   TRUE on success
   */
  protected function export(): bool {
    try {
      // This event is triggered by lots of CRUD operations. Reuse the same
      // exporter if this event is called more than once in the same thread.
      if (!$this->exporter) {
        $this->exporter = $this->customExporterPluginManager->createInstance('language-config');
      }

      $config = $this->configFactory->get('static_export.settings');

      // If running on CLI, check if a export operation must be executed.
      if ($this->staticSuiteUtils->isRunningOnCli() && !$config->get('exportable_config.export_when_crud_happens_on_cli')) {
        return FALSE;
      }

      $requestBuild = TRUE;
      // If running on CLI, check if a build must be requested.
      if ($this->staticSuiteUtils->isRunningOnCli()) {
        $requestBuild = !$config->get('exportable_config.request_build_when_crud_exports_on_cli');
      }
      $this->exporter->setMustBuild($requestBuild);
      $fileCollectionGroup = $this->exporter->write();
      $this->messenger->addFileCollectionGroup($fileCollectionGroup);
    }
    catch (StaticSuiteUserException | PluginException $e) {
      $this->messenger->addError($e->getMessage());
    }
    catch (Throwable $e) {
      $this->messenger->addException($e);
    }

    return TRUE;
  }

  /**
   * Tells whether a configuration name affects language data.
   *
   * @param string $configName
   *   Configuration name.
   *
   * @return bool
   *   True if affects language data, false otherwise.
   */
  protected static function affectsLanguages(string $configName): bool {
    return (
      $configName === 'language.negotiation' ||
      $configName === 'system.site' ||
      $configName === 'language.types' ||
      strpos($configName, 'language.entity.') === 0
    );
  }

}
