<?php

namespace Drupal\static_export\EventSubscriber;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\locale\LocaleEvent;
use Drupal\locale\LocaleEvents;
use Drupal\static_export\Messenger\Messenger;
use Drupal\static_export\Exporter\Type\Locale\LocaleExporterPluginManagerInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Throwable;

/**
 * Event subscriber for locales.
 */
class LocaleEventSubscriber implements EventSubscriberInterface {

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Locale exporter manager.
   *
   * @var \Drupal\static_export\Exporter\Type\Locale\LocaleExporterPluginManagerInterface
   */
  protected $localeExporterManager;

  /**
   * Messenger service.
   *
   * @var \Drupal\static_export\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Static Suite Utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * The exporter that will handle the event.
   *
   * @var \Drupal\static_export\Exporter\ExporterPluginInterface
   */
  protected $exporter;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal config factory.
   * @param \Drupal\static_export\Exporter\Type\Locale\LocaleExporterPluginManagerInterface $localeExporterManager
   *   Exporter Manager.
   * @param \Drupal\static_export\Messenger\Messenger $messenger
   *   Messenger service.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $static_suite_utils
   *   Static Suite Utils.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LocaleExporterPluginManagerInterface $localeExporterManager, Messenger $messenger, StaticSuiteUtilsInterface $static_suite_utils) {
    $this->configFactory = $config_factory;
    $this->localeExporterManager = $localeExporterManager;
    $this->messenger = $messenger;
    $this->staticSuiteUtils = $static_suite_utils;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[LocaleEvents::SAVE_TRANSLATION][] = ['onSaveLocale'];
    return $events;
  }

  /**
   * Reacts to a save event.
   *
   * @param \Drupal\locale\LocaleEvent $event
   *   A locale save event.
   *
   * @return bool
   *   TRUE on success
   */
  public function onSaveLocale(LocaleEvent $event): bool {
    // @todo - add support for exporting multiple files at once.
    $langCodes = $event->getLangCodes();

    // Multiple langcodes are possible. Export the first one as the master
    // language, and the other ones as slaves.
    $masterLangCode = array_shift($langCodes);
    // If running on CLI, check if a export operation must be executed.
    if ($this->staticSuiteUtils->isRunningOnCli() && !$this->configFactory->get('static_export.settings')->get('exportable_locale.export_when_crud_happens_on_cli')) {
      return FALSE;
    }

    $requestBuild = TRUE;
    // If running on CLI, check if a build must be requested.
    if ($this->staticSuiteUtils->isRunningOnCli()) {
      $requestBuild = !$this->configFactory->get('static_export.settings')->get('exportable_locale.request_build_when_crud_exports_on_cli');
    }

    // Export the master language.
    $exporter = NULL;
    try {
      $exporter = $this->localeExporterManager->getDefaultInstance();
      $exporter->setMustBuild($requestBuild);
      $fileCollectionGroup = $exporter->export([
        'language' => $masterLangCode,
      ]);
      $this->messenger->addFileCollectionGroup($fileCollectionGroup);
    }
    catch (StaticSuiteUserException | PluginException $e) {
      $this->messenger->addError($e->getMessage());
    }
    catch (Throwable $e) {
      $this->messenger->addException($e);
    }

    return TRUE;
  }

}
