<?php

namespace Drupal\static_export\Exporter\Data\Includes\Loader;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\static_export\Exporter\Output\Uri\UriFactoryInterface;
use Drupal\static_export\Exporter\Output\Uri\UriInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an abstract data include loader.
 */
abstract class DataIncludeLoaderPluginBase extends PluginBase implements DataIncludeLoaderPluginInterface, ContainerFactoryPluginInterface {

  /**
   * Loaded includes.
   *
   * @var array
   */
  protected $loadedIncludes = [];

  /**
   * The URI factory.
   *
   * @var \Drupal\static_export\Exporter\Output\Uri\UriFactoryInterface
   */
  protected $uriFactory;

  /**
   * Constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\static_export\Exporter\Output\Uri\UriFactoryInterface $uriFactory
   *   The URI factory.
   */
  public function __construct(array $configuration,
                              string $plugin_id,
                              $plugin_definition,
                              UriFactoryInterface $uriFactory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->uriFactory = $uriFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get("static_export.uri_factory"),
    );
  }

  /**
   * Tells whether this data is supported.
   *
   * Data can arrive from different sources (GraphQL, JSON:API, custom
   * exporters, etc) so we check if data is in a format this loader understand.
   *
   * @param string $data
   *   The string to be parsed.
   *
   * @return bool
   *   True if data is supported
   */
  abstract protected function isDataSupported(string $data): bool;

  /**
   * Find includes inside data.
   *
   * @param string $data
   *   Data to be parsed.
   *
   * @return array|null
   *   Array of matches, or null if nothing found.
   */
  abstract protected function findIncludes(string $data): ?array;

  /**
   * {@inheritdoc}
   */
  public function load(string $data): string {
    // Load includes only for known data.
    if (!$this->isDataSupported($data)) {
      return $data;
    }

    // Ensure $this->loadedIncludes is empty on each execution.
    $this->loadedIncludes = [];
    $data = $this->parseData($data);
    $this->loadedIncludes = [];

    return $data;
  }

  /**
   * Parse data and load includes.
   *
   * We use a helper function so we can manage $this->loadedIncludes in a more
   * easier way.
   *
   * @param string $data
   *   A string to be parsed.
   *
   * @return string
   *   The parsed string with includes loaded.
   */
  abstract protected function parseData(string $data): string;

  /**
   * Loads an include and mark it as loaded to avoid repeating it.
   *
   * Internally it uses isIncludeAlreadyLoaded() and markIncludeAsLoaded().
   *
   * @param \Drupal\static_export\Exporter\Output\Uri\UriInterface $uri
   *   Th inclde to be loaded.
   *
   * @return string|null
   *   A string with includes loaded, or NULL if nothing found or include
   *   already loaded.
   */
  protected function loadInclude(UriInterface $uri): ?string {
    // Avoid loading an include twice.
    if ($this->isIncludeAlreadyLoaded($uri)) {
      $includeData = NULL;
    }
    else {
      $this->markIncludeAsLoaded($uri);
      $includeData = trim(@file_get_contents($uri));
      $includeData = $includeData ?? NULL;
    }
    return $includeData;
  }

  /**
   * Tells whether a fragment has been already loaded.
   *
   * GraphQL fragments can not be repeated.
   *
   * @param \Drupal\static_export\Exporter\Output\Uri\UriInterface $uri
   *   The file to check.
   *
   * @return bool
   *   True if it has been already loaded, false otherwise.
   */
  protected function isIncludeAlreadyLoaded(UriInterface $uri): bool {
    return isset($this->loadedIncludes[$uri->getComposed()]);
  }

  /**
   * Marks a fragment as already loaded.
   *
   * @param \Drupal\static_export\Exporter\Output\Uri\UriInterface $uri
   *   The file to be marked as already loaded.
   */
  protected function markIncludeAsLoaded(UriInterface $uri): void {
    $this->loadedIncludes[$uri->getComposed()] = TRUE;
  }

}
