<?php

namespace Drupal\static_export\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\static_export\Exporter\Type\Config\ConfigExporterPluginManagerInterface;
use Drupal\static_export\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for adding exportable configurations.
 */
class ExportableConfigAddForm extends ConfigFormBase {

  /**
   * The config exporter manager.
   *
   * @var \Drupal\static_export\Exporter\Type\Config\ConfigExporterPluginManagerInterface
   */
  protected $configExporterManager;

  /**
   * The static exporter messenger.
   *
   * @var \Drupal\static_export\Messenger\MessengerInterface
   */
  protected $staticExportMessenger;

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\static_export\Exporter\Type\Config\ConfigExporterPluginManagerInterface $configExporterManager
   *   The config exporter manager.
   * @param \Drupal\static_export\Messenger\MessengerInterface $staticExportMessenger
   *   The static exporter messenger.
   */
  public function __construct(ConfigFactoryInterface $configFactory, ConfigExporterPluginManagerInterface $configExporterManager, MessengerInterface $staticExportMessenger) {
    parent::__construct($configFactory);
    $this->configExporterManager = $configExporterManager;
    $this->staticExportMessenger = $staticExportMessenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.static_config_exporter'),
      $container->get('static_export.messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_export_exportable_config_add';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_export.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $currentConfigObjects = $config->get('exportable_config.objects_to_export');

    $allConfigNames = $this->configFactory->listAll();
    foreach ($allConfigNames as $configName) {
      if (!is_array($currentConfigObjects) || !in_array($configName, $currentConfigObjects, TRUE)) {
        $options[$configName] = $configName;
      }
    }

    $form['object_to_export'] = [
      '#type' => 'select',
      '#options' => $options,
      '#title' => $this->t('Select configuration object to export'),
      '#required' => TRUE,
      '#suffix' => '<p>' . $this->t('This action will export the configuration object of your choice and could potentially trigger a build.') . '</p>',
    ];

    $form = parent::buildForm($form, $form_state);
    $form['actions']['submit']['#value'] = $this->t('Add');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $currentConfigObjects = $config->get('exportable_config.objects_to_export');

    if (is_array($currentConfigObjects) && in_array($form_state->getValue('object_to_export'), $currentConfigObjects, TRUE)) {
      $form_state->setErrorByName(
        'data_dir',
        $this->t('Configuration "@config" is already exported.', ['@config', $form_state->getValue('object_to_export')]));
    }
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteException
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $configObjects = $config->get('exportable_config.objects_to_export');
    $configObjects[] = $form_state->getValue('object_to_export');
    sort($configObjects);
    $config
      ->set('exportable_config.objects_to_export', array_values($configObjects))
      ->save();

    // Tell the config exporter to export this object data.
    $fileCollectionGroup = $this->configExporterManager->getDefaultInstance()->export(['name' => $form_state->getValue('object_to_export')]);
    $this->staticExportMessenger->addFileCollectionGroup($fileCollectionGroup);

    $this->messenger()->addMessage($this->t('The configuration %label has been saved.', [
      '%label' => $form_state->getValue('object_to_export'),
    ]));

    $form_state->setRedirect('static_export.exportable_config.collection');
  }

}
