<?php

namespace Drupal\static_export\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for Static Export.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_export_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_export.settings'];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');

    $form['data'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Data directory'),
    ];

    $form['data']['data_dir'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Data directory path'),
      '#required' => TRUE,
      '#description' => $this->t('Exported data is saved in the above path. It must start with a leading slash. It should be writable by the user running your web-server (usually <em>www-data</em> or similar). Relative to <em>DRUPAL_ROOT</em> (@drupal_root). It can be set outside <em>DRUPAL_ROOT</em> using "../"', ['@drupal_root' => DRUPAL_ROOT]),
      '#default_value' => $config->get('data_dir'),
    ];

    $form['log'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Logs'),
    ];

    $form['log']['log_dir'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Log directory path'),
      '#required' => TRUE,
      '#description' => $this->t('Logs for each export operation are saved into individual files in the above path. It must start with a leading slash. It should be writable by the user running your web-server (usually <em>www-data</em> or similar). Relative to <em>DRUPAL_ROOT</em> (@drupal_root). It can be set outside <em>DRUPAL_ROOT</em> using "../". A recommended value is the value from above field "Data directory path", plus ".work/log".', ['@drupal_root' => DRUPAL_ROOT]),
      '#default_value' => $config->get('log_dir'),
    ];

    $form['log']['max_days_to_keep_logs'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum days to keep log files'),
      '#required' => TRUE,
      '#description' => $this->t('Old logs will be automatically deleted when Drupal cron runs.'),
      '#default_value' => $config->get('max_days_to_keep_logs'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $dataDir = $form_state->getValue('data_dir');
    if (strpos($dataDir, '/') !== 0) {
      $form_state->setErrorByName(
        'data_dir',
        $this->t('Data directory path must start with a leading slash.'));
    }

    $logDir = $form_state->getValue('log_dir');
    if (strpos($logDir, '/') !== 0) {
      $form_state->setErrorByName(
        'log_dir',
        $this->t('Log directory path must start with a leading slash.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $config
      ->set('data_dir', preg_replace("/\/$/", "", $form_state->getValue('data_dir')))
      ->set('log_dir', preg_replace("/\/$/", "", $form_state->getValue('log_dir')))
      ->set('max_days_to_keep_logs', (int) $form_state->getValue('max_days_to_keep_logs'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
