<?php

namespace Drupal\static_preview_gatsby_instant\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface;
use Drupal\static_suite\Entity\EntityUtils;
use Drupal\static_export\Event\StaticExportEvent;
use Drupal\static_export\Event\StaticExportEvents;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for Static Build.
 */
class EventSubscriber implements EventSubscriberInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Static Builder Manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface
   */
  protected $staticBuilderPluginManager;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtils
   */
  protected $entityUtils;

  /**
   * Static Export utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * Constructs the subscriber.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface $staticBuilderPluginManager
   *   Static Builder Manager.
   * @param \Drupal\static_suite\Entity\EntityUtils $entityUtils
   *   Utils for working with entities.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $static_suite_utils
   *   Static Export utils.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StaticBuilderPluginManagerInterface $staticBuilderPluginManager, EntityUtils $entityUtils, StaticSuiteUtilsInterface $static_suite_utils) {
    $this->configFactory = $config_factory;
    $this->staticBuilderPluginManager = $staticBuilderPluginManager;
    $this->entityUtils = $entityUtils;
    $this->staticSuiteUtils = $static_suite_utils;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[StaticExportEvents::WRITE_QUEUE_PROCESSING_ENDS][] = ['triggerPreviewBuild'];
    return $events;
  }

  /**
   * Reacts to a StaticExportEvents::CHECK_PARAMS_ENDS event.
   *
   * @param \Drupal\static_export\Event\StaticExportEvent $event
   *   The Static Export event.
   *
   * @return \Drupal\static_export\Event\StaticExportEvent
   *   The processed event.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function triggerPreviewBuild(StaticExportEvent $event) {
    $exporter = $event->getExporter();

    // Check if received event should request a build.
    if (!$exporter->mustBuild()) {
      $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_PREVIEW . ' | Exporter is not allowed to build. No build triggered.');
      return $event;
    }

    // Check if received FileCollections contains changes.
    $fileCollectionGroup = $event->getFileCollectionGroup();
    if (!$fileCollectionGroup || !$fileCollectionGroup->isAnyFileCollectionExecuted()) {
      $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_PREVIEW . ' | No changes detected on received FileCollectionGroup. No build triggered.');
      return $event;
    }

    // Check if received FileCollections matches paths that request a build.
    $executedFilePaths = $fileCollectionGroup->getExecutedFilePathsFromAllFileCollections();
    $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_PREVIEW . " | Changed files on received FileCollectionGroup:\n" . implode("\n", $executedFilePaths));
    $changedFilesRegExpList = $this->getBuildTriggerRegExpList();
    $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_PREVIEW . " | RegExp List to match changed files:\n" . implode("\n", $changedFilesRegExpList));
    if (is_array($changedFilesRegExpList) && count($changedFilesRegExpList) && !$this->staticSuiteUtils->isAnyItemMatchingRegexpList($executedFilePaths, $changedFilesRegExpList)) {
      $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_PREVIEW . ' | No changed files match RegExp List. No build triggered.');
      return $event;
    }

    $event->getExporter()->logMessage('[build] ' . StaticBuilderPluginInterface::RUN_MODE_PREVIEW . ' | Some changed files match RegExp List. Build triggered.');

    $plugin = $this->staticBuilderPluginManager->getInstance([
      'plugin_id' => 'gatsby',
      'configuration' => [
        'run-mode' => StaticBuilderPluginInterface::RUN_MODE_PREVIEW,
        // We should honor live lock, because this build must run when
        // something published has changed. Honoring the preview lock would
        // trigger a build only when unpublished data has changed.
        'lock-mode' => StaticBuilderPluginInterface::LOCK_MODE_LIVE,
      ],
    ]);

    $plugin->init();

    return $event;
  }

  /**
   * Get build_trigger_regexp_list config value.
   *
   * @return array
   *   Array with Regular Expressions or null otherwise.
   */
  protected function getBuildTriggerRegExpList(): array {
    $config = $this->configFactory->get('static_build.settings');
    $regExpList = $config->get('build_trigger_regexp_list_preview');
    if (is_array($regExpList)) {
      return $regExpList;
    }
    return [];
  }

}
