<?php

namespace Drupal\static_preview_gatsby_instant\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\static_build\Plugin\StaticBuilderPluginInterface;
use Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface;
use Drupal\static_build\Release\ReleaseManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for Static Preview - Gatsby - Instant.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Release manager.
   *
   * @var \Drupal\static_build\Release\ReleaseManagerInterface
   */
  protected $releaseManager;

  /**
   * Static builder manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface
   */
  protected $staticBuilderPluginManager;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\static_build\Plugin\StaticBuilderPluginManagerInterface $staticBuilderPluginManager
   *   Static builder manager.
   * @param \Drupal\static_build\Release\ReleaseManagerInterface $releaseManager
   *   Release manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StaticBuilderPluginManagerInterface $staticBuilderPluginManager, ReleaseManagerInterface $releaseManager) {
    parent::__construct($config_factory);
    $this->staticBuilderPluginManager = $staticBuilderPluginManager;
    $this->releaseManager = $releaseManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.static_builder'),
      $container->get('static_build.release_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_preview_gatsby_instant_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_preview_gatsby_instant.settings'];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $builder = $this->staticBuilderPluginManager->getInstance([
      'plugin_id' => 'gatsby',
      'configuration' => [
        'run-mode' => StaticBuilderPluginInterface::RUN_MODE_PREVIEW,
      ],
    ]);
    $this->releaseManager = $builder->getReleaseManager();
    $previewDir = $this->releaseManager->getCurrentPointerPath();

    $config = $this->config('static_preview_gatsby_instant.settings');

    $form['preview_component_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Preview component path'),
      '#required' => TRUE,
      '#description' => $this->t("It must start with a leading slash. Path to your Gatsby preview component, e.g.- /preview, inside Gatsby's public folder ('%previewDir'). Please refer to the docs for guidance on what should that component do.", ['%previewDir' => $previewDir]),
      '#default_value' => $config->get('preview_component_path'),
    ];

    $form['build_trigger_regexp_list'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Regular Expressions to detect changed files that trigger a "@preview" rebuild', ['@preview' => StaticBuilderPluginInterface::RUN_MODE_PREVIEW]),
      '#required' => FALSE,
      '#description' => $this->t(
        "If a global part of your site is built with content coming from Drupal (e.g.- a global header or footer) it won't be updated unless a build is run. You should define, inside <a href='@static_build_config_form_link'>Static Build module's configuration form</a>, which files must trigger a '@preview' build.",
        [
          '@static_build_config_form_link' => Url::fromRoute('static_build.settings')->toString(),
          '@preview' => StaticBuilderPluginInterface::RUN_MODE_PREVIEW,
        ]),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $previewPath = $form_state->getValue('preview_component_path');
    if (strpos($previewPath, '/') !== 0) {
      $form_state->setErrorByName(
        'preview_component_path',
        $this->t('Preview component path must start with a leading slash.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_preview_gatsby_instant.settings');
    $config
      ->set('preview_component_path', preg_replace("/\/$/", "", $form_state->getValue('preview_component_path')))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
