<?php

namespace Drupal\static_suite\Language;

use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\language\ConfigurableLanguageManagerInterface;

/**
 * Simple service to execute a callable in a defined language context.
 *
 * @see GraphQLLanguageContext, based on it.
 */
class LanguageContext implements LanguageContextInterface {

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Externally overridable language negotiator.
   *
   * @var \Drupal\static_suite\Language\OverridableLanguageNegotiatorInterface
   */
  protected $languageNegotiator;

  /**
   * GraphQLLanguageContext constructor.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager service.
   * @param \Drupal\static_suite\Language\OverridableLanguageNegotiatorInterface $languageNegotiator
   *   Externally overridable language negotiator.
   */
  public function __construct(LanguageManagerInterface $languageManager, OverridableLanguageNegotiatorInterface $languageNegotiator) {
    $this->languageManager = $languageManager;
    $this->languageNegotiator = $languageNegotiator;
  }

  /**
   * {@inheritdoc}
   *
   * @noinspection PhpMissingReturnTypeInspection
   */
  public function executeInLanguageContext(callable $callable, string $langcode) {
    $originalLanguage = $this->languageManager->getCurrentLanguage();
    $originalLangcode = $originalLanguage->getId();
    $newLanguage = $this->languageManager->getLanguage($langcode);
    $originalNegotiator = NULL;
    if ($newLanguage && ($originalLangcode !== $langcode) && $this->languageManager instanceof ConfigurableLanguageManagerInterface) {
      $originalNegotiator = $this->languageManager->getNegotiator();
      $this->languageManager->reset();
      $this->languageNegotiator->setLanguageCode($langcode);
      $this->languageManager->setNegotiator($this->languageNegotiator);
      $this->languageManager->setConfigOverrideLanguage($newLanguage);
    }

    // Execute the callable.
    try {
      return $callable();
    }
    finally {
      // In any case, set the language back to its previous state.
      if (($originalLangcode !== $langcode) && $this->languageManager instanceof ConfigurableLanguageManagerInterface) {
        $this->languageManager->reset();
        if ($originalNegotiator) {
          $this->languageManager->setNegotiator($originalNegotiator);
          $this->languageManager->setConfigOverrideLanguage($originalLanguage);
        }
      }
    }
  }

}
